@@ -176,6 +176,29 @@ private:
     LibInput::Device *m_device;
 };
 
+class TabletEvent : public QTabletEvent
+{
+public:
+    TabletEvent(Type t, const QPointF &pos, const QPointF &globalPos,
+                int device, int pointerType, qreal pressure, int xTilt, int yTilt,
+                qreal tangentialPressure, qreal rotation, int z,
+                Qt::KeyboardModifiers keyState, qint64 uniqueID,
+                Qt::MouseButton button, Qt::MouseButtons buttons, InputRedirection::TabletToolType toolType,
+                const QVector<InputRedirection::Capability> &capabilities,
+                quint64 serialId, const QString &tabletSysname);
+
+    InputRedirection::TabletToolType toolType() const { return m_toolType; }
+    QVector<InputRedirection::Capability> capabilities() const { return m_capabilities; }
+    quint64 serialId() const { return m_serialId; }
+    QString tabletSysName() { return m_tabletSysName; }
+
+private:
+    const InputRedirection::TabletToolType m_toolType;
+    const QVector<InputRedirection::Capability> m_capabilities;
+    const quint64 m_serialId;
+    const QString m_tabletSysName;
+};
+
 }
 
 #endif
