@@ -31,6 +31,7 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 Q_DECLARE_METATYPE(NET::WindowType)
 
 static const QByteArray s_skipClosePropertyName = QByteArrayLiteral("KWIN_SKIP_CLOSE_ANIMATION");
+static const QByteArray s_shadowEnabledPropertyName = QByteArrayLiteral("kwin_shadow_enabled");
 
 namespace KWin
 {
@@ -62,6 +63,7 @@ InternalClient::InternalClient(QWindow *window)
     setOpacity(m_internalWindow->opacity());
     setSkipCloseAnimation(m_internalWindow->property(s_skipClosePropertyName).toBool());
 
+    // Create scene window, effect window, and update server-side shadow.
     setupCompositing();
     updateColorScheme();
 
@@ -86,6 +88,9 @@ bool InternalClient::eventFilter(QObject *watched, QEvent *event)
         if (pe->propertyName() == s_skipClosePropertyName) {
             setSkipCloseAnimation(m_internalWindow->property(s_skipClosePropertyName).toBool());
         }
+        if (pe->propertyName() == s_shadowEnabledPropertyName) {
+            updateShadow();
+        }
         if (pe->propertyName() == "kwin_windowType") {
             m_windowType = m_internalWindow->property("kwin_windowType").value<NET::WindowType>();
             workspace()->updateClientArea();
@@ -139,6 +144,16 @@ QSize InternalClient::clientSize() const
     return m_clientSize;
 }
 
+QSize InternalClient::minSize() const
+{
+    return m_internalWindow->minimumSize();
+}
+
+QSize InternalClient::maxSize() const
+{
+    return m_internalWindow->maximumSize();
+}
+
 void InternalClient::debug(QDebug &stream) const
 {
     stream.nospace() << "\'InternalClient:" << m_internalWindow << "\'";
@@ -203,26 +218,6 @@ bool InternalClient::isCloseable() const
     return true;
 }
 
-bool InternalClient::isFullScreenable() const
-{
-    return false;
-}
-
-bool InternalClient::isFullScreen() const
-{
-    return false;
-}
-
-bool InternalClient::isMaximizable() const
-{
-    return false;
-}
-
-bool InternalClient::isMinimizable() const
-{
-    return false;
-}
-
 bool InternalClient::isMovable() const
 {
     return true;
@@ -287,16 +282,6 @@ quint32 InternalClient::windowId() const
     return m_windowId;
 }
 
-MaximizeMode InternalClient::maximizeMode() const
-{
-    return MaximizeRestore;
-}
-
-QRect InternalClient::geometryRestore() const
-{
-    return m_maximizeRestoreGeometry;
-}
-
 bool InternalClient::isShown(bool shaded_is_shown) const
 {
     Q_UNUSED(shaded_is_shown)
@@ -314,27 +299,18 @@ void InternalClient::hideClient(bool hide)
     Q_UNUSED(hide)
 }
 
-void InternalClient::resizeWithChecks(int w, int h, ForceGeometry_t force)
+void InternalClient::resizeWithChecks(const QSize &size, ForceGeometry_t force)
 {
     Q_UNUSED(force)
     if (!m_internalWindow) {
         return;
     }
-    QRect area = workspace()->clientArea(WorkArea, this);
-    // don't allow growing larger than workarea
-    if (w > area.width()) {
-        w = area.width();
-    }
-    if (h > area.height()) {
-        h = area.height();
-    }
-    setFrameGeometry(QRect(x(), y(), w, h));
+    const QRect area = workspace()->clientArea(WorkArea, this);
+    setFrameGeometry(QRect{pos(), size.boundedTo(area.size())}, force);
 }
 
-void InternalClient::setFrameGeometry(int x, int y, int w, int h, ForceGeometry_t force)
+void InternalClient::setFrameGeometry(const QRect &rect, ForceGeometry_t force)
 {
-    const QRect rect(x, y, w, h);
-
     if (areGeometryUpdatesBlocked()) {
         m_frameGeometry = rect;
         if (pendingGeometryUpdate() == PendingGeometryForced) {
@@ -365,11 +341,6 @@ void InternalClient::setFrameGeometry(int x, int y, int w, int h, ForceGeometry_
     }
 }
 
-void InternalClient::setGeometryRestore(const QRect &rect)
-{
-    m_maximizeRestoreGeometry = rect;
-}
-
 bool InternalClient::supportsWindowRules() const
 {
     return false;
@@ -388,20 +359,8 @@ void InternalClient::setOnAllActivities(bool set)
     // Internal clients do not support activities.
 }
 
-bool InternalClient::takeFocus()
+void InternalClient::takeFocus()
 {
-    return false;
-}
-
-bool InternalClient::userCanSetFullScreen() const
-{
-    return false;
-}
-
-void InternalClient::setFullScreen(bool set, bool user)
-{
-    Q_UNUSED(set)
-    Q_UNUSED(user)
 }
 
 void InternalClient::setNoBorder(bool set)
@@ -478,7 +437,7 @@ void InternalClient::present(const QSharedPointer<QOpenGLFramebufferObject> fbo)
 
     const QSize bufferSize = fbo->size() / bufferScale();
 
-    commitGeometry(QRect(pos(), sizeForClientSize(bufferSize)));
+    commitGeometry(QRect(pos(), clientSizeToFrameSize(bufferSize)));
     markAsMapped();
 
     if (m_internalFBO != fbo) {
@@ -497,7 +456,7 @@ void InternalClient::present(const QImage &image, const QRegion &damage)
 
     const QSize bufferSize = image.size() / bufferScale();
 
-    commitGeometry(QRect(pos(), sizeForClientSize(bufferSize)));
+    commitGeometry(QRect(pos(), clientSizeToFrameSize(bufferSize)));
     markAsMapped();
 
     if (m_internalImage.size() != image.size()) {
@@ -528,26 +487,6 @@ bool InternalClient::belongsToSameApplication(const AbstractClient *other, SameA
     return qobject_cast<const InternalClient *>(other) != nullptr;
 }
 
-void InternalClient::changeMaximize(bool horizontal, bool vertical, bool adjust)
-{
-    Q_UNUSED(horizontal)
-    Q_UNUSED(vertical)
-    Q_UNUSED(adjust)
-
-    // Internal clients are not maximizable.
-}
-
-void InternalClient::destroyDecoration()
-{
-    if (!isDecorated()) {
-        return;
-    }
-
-    const QRect clientGeometry = frameRectToClientRect(frameGeometry());
-    AbstractClient::destroyDecoration();
-    setFrameGeometry(clientGeometry);
-}
-
 void InternalClient::doMove(int x, int y)
 {
     Q_UNUSED(x)
@@ -578,32 +517,6 @@ void InternalClient::updateCaption()
     }
 }
 
-void InternalClient::createDecoration(const QRect &rect)
-{
-    KDecoration2::Decoration *decoration = Decoration::DecorationBridge::self()->createDecoration(this);
-    if (decoration) {
-        QMetaObject::invokeMethod(decoration, "update", Qt::QueuedConnection);
-        connect(decoration, &KDecoration2::Decoration::shadowChanged, this, &Toplevel::updateShadow);
-        connect(decoration, &KDecoration2::Decoration::bordersChanged, this,
-            [this]() {
-                GeometryUpdatesBlocker blocker(this);
-                const QRect oldGeometry = frameGeometry();
-                if (!isShade()) {
-                    checkWorkspacePosition(oldGeometry);
-                }
-                emit geometryShapeChanged(this, oldGeometry);
-            }
-        );
-    }
-
-    const QRect oldFrameGeometry = frameGeometry();
-
-    setDecoration(decoration);
-    setFrameGeometry(clientRectToFrameRect(rect));
-
-    emit geometryShapeChanged(this, oldFrameGeometry);
-}
-
 void InternalClient::requestGeometry(const QRect &rect)
 {
     if (m_internalWindow) {
@@ -624,9 +537,11 @@ void InternalClient::commitGeometry(const QRect &rect)
     addWorkspaceRepaint(visibleRect());
     syncGeometryToInternalWindow();
 
-    const QRect oldGeometry = frameGeometryBeforeUpdateBlocking();
+    if (frameGeometryBeforeUpdateBlocking() != frameGeometry()) {
+        emit frameGeometryChanged(this, frameGeometryBeforeUpdateBlocking());
+    }
+    emit geometryShapeChanged(this, frameGeometryBeforeUpdateBlocking());
     updateGeometryBeforeUpdateBlocking();
-    emit geometryShapeChanged(this, oldGeometry);
 
     if (isResize()) {
         performMoveResize();
