@@ -43,6 +43,8 @@ public:
     QString captionSuffix() const override;
     QPoint clientContentPos() const override;
     QSize clientSize() const override;
+    QSize minSize() const override;
+    QSize maxSize() const override;
     void debug(QDebug &stream) const override;
     QRect transparentRect() const override;
     NET::WindowType windowType(bool direct = false, int supported_types = 0) const override;
@@ -53,10 +55,6 @@ public:
     QByteArray windowRole() const override;
     void closeWindow() override;
     bool isCloseable() const override;
-    bool isFullScreenable() const override;
-    bool isFullScreen() const override;
-    bool isMaximizable() const override;
-    bool isMinimizable() const override;
     bool isMovable() const override;
     bool isMovableAcrossScreens() const override;
     bool isResizable() const override;
@@ -68,28 +66,21 @@ public:
     bool isInputMethod() const override;
     bool isOutline() const override;
     quint32 windowId() const override;
-    MaximizeMode maximizeMode() const override;
-    QRect geometryRestore() const override;
     bool isShown(bool shaded_is_shown) const override;
     bool isHiddenInternal() const override;
     void hideClient(bool hide) override;
-    using AbstractClient::resizeWithChecks;
-    void resizeWithChecks(int w, int h, ForceGeometry_t force = NormalGeometrySet) override;
-    using AbstractClient::setFrameGeometry;
-    void setFrameGeometry(int x, int y, int w, int h, ForceGeometry_t force = NormalGeometrySet) override;
-    void setGeometryRestore(const QRect &rect) override;
+    void resizeWithChecks(const QSize &size, ForceGeometry_t force = NormalGeometrySet) override;
+    void setFrameGeometry(const QRect &rect, ForceGeometry_t force = NormalGeometrySet) override;
     bool supportsWindowRules() const override;
     AbstractClient *findModal(bool allow_itself = false) override;
     void setOnAllActivities(bool set) override;
-    bool takeFocus() override;
-    bool userCanSetFullScreen() const override;
-    void setFullScreen(bool set, bool user = true) override;
+    void takeFocus() override;
     void setNoBorder(bool set) override;
     void updateDecoration(bool check_workspace_pos, bool force = false) override;
     void updateColorScheme() override;
     void showOnScreenEdge() override;
+    void destroyClient() override;
 
-    void destroyClient();
     void present(const QSharedPointer<QOpenGLFramebufferObject> fbo);
     void present(const QImage &image, const QRegion &damage);
     QWindow *internalWindow() const;
@@ -97,14 +88,11 @@ public:
 protected:
     bool acceptsFocus() const override;
     bool belongsToSameApplication(const AbstractClient *other, SameApplicationChecks checks) const override;
-    void changeMaximize(bool horizontal, bool vertical, bool adjust) override;
-    void destroyDecoration() override;
     void doMove(int x, int y) override;
     void doResizeSync() override;
     void updateCaption() override;
 
 private:
-    void createDecoration(const QRect &rect);
     void requestGeometry(const QRect &rect);
     void commitGeometry(const QRect &rect);
     void setCaption(const QString &caption);
@@ -113,7 +101,6 @@ private:
     void updateInternalWindowGeometry();
 
     QWindow *m_internalWindow = nullptr;
-    QRect m_maximizeRestoreGeometry;
     QSize m_clientSize = QSize(0, 0);
     QString m_captionNormal;
     QString m_captionSuffix;
