@@ -93,7 +93,6 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #include "effects.h"
 #include "composite.h"
 #include "screenedge.h"
-#include "xdgshellclient.h"
 #include "wayland_server.h"
 #include "internal_client.h"
 
@@ -125,7 +124,6 @@ void Workspace::updateStackingOrder(bool propagate_new_clients)
     stacking_order = new_stacking_order;
     if (changed || propagate_new_clients) {
         propagateClients(propagate_new_clients);
-        markXStackingOrderAsDirty();
         emit stackingOrderChanged();
         if (m_compositor) {
             m_compositor->addRepaintFull();
@@ -223,14 +221,20 @@ void Workspace::propagateClients(bool propagate_new_clients)
     cl = new xcb_window_t[ manual_overlays.count() + stacking_order.count()];
     pos = 0;
     for (auto it = stacking_order.constBegin(); it != stacking_order.constEnd(); ++it) {
-        if ((*it)->isClient())
-            cl[pos++] = (*it)->window();
+        X11Client *client = qobject_cast<X11Client *>(*it);
+        if (client) {
+            cl[pos++] = client->window();
+        }
     }
     for (const auto win : manual_overlays) {
         cl[pos++] = win;
     }
     rootInfo()->setClientListStacking(cl, pos);
     delete [] cl;
+
+    // Make the cached stacking order invalid here, in case we need the new stacking order before we get
+    // the matching event, due to X being asynchronous.
+    markXStackingOrderAsDirty();
 }
 
 /**
@@ -505,7 +509,7 @@ QList<Toplevel *> Workspace::constrainedStackingOrder()
     QList<Toplevel *> layer[ NumLayers ];
 
     // build the order from layers
-    QVector< QMap<Group*, Layer> > minimum_layer(screens()->count());
+    QVector< QMultiMap<Group*, Layer> > minimum_layer(screens()->count());
     for (auto it = unconstrained_stacking_order.constBegin(),
                                   end = unconstrained_stacking_order.constEnd(); it != end; ++it) {
         Layer l = (*it)->layer();
@@ -521,7 +525,7 @@ QList<Toplevel *> Workspace::constrainedStackingOrder()
                 l = ActiveLayer;
             *mLayer = l;
         } else if (c) {
-            minimum_layer[screen].insertMulti(c->group(), l);
+            minimum_layer[screen].insert(c->group(), l);
         }
         layer[ l ].append(*it);
     }
@@ -848,14 +852,6 @@ void X11Client::restackWindow(xcb_window_t above, int detail, NET::RequestSource
         sendSyntheticConfigureNotify();
 }
 
-void X11Client::doSetKeepAbove()
-{
-}
-
-void X11Client::doSetKeepBelow()
-{
-}
-
 bool X11Client::belongsToDesktop() const
 {
     foreach (const X11Client *c, group()->members()) {
