@@ -24,14 +24,14 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #include "drm_object_connector.h"
 
 #include "composite.h"
+#include "cursor.h"
 #include "logind.h"
 #include "logging.h"
 #include "main.h"
-#include "orientation_sensor.h"
 #include "screens_drm.h"
 #include "wayland_server.h"
 // KWayland
-#include <KWayland/Server/output_interface.h>
+#include <KWaylandServer/output_interface.h>
 // KF5
 #include <KConfigGroup>
 #include <KLocalizedString>
@@ -162,7 +162,7 @@ QMatrix4x4 DrmOutput::matrixDisplay(const QSize &s) const
         const QSize center = s / 2;
 
         matrix.translate(center.width(), center.height());
-        matrix.rotate(angle, 0, 0, 1);
+        matrix.rotate(-angle, 0, 0, 1);
         matrix.translate(-center.width(), -center.height());
     }
     matrix.scale(scale());
@@ -171,10 +171,7 @@ QMatrix4x4 DrmOutput::matrixDisplay(const QSize &s) const
 
 void DrmOutput::updateCursor()
 {
-    if (m_deleted) {
-        return;
-    }
-    QImage cursorImage = m_backend->softwareCursor();
+    QImage cursorImage = Cursors::self()->currentCursor()->image();
     if (cursorImage.isNull()) {
         return;
     }
@@ -189,11 +186,12 @@ void DrmOutput::updateCursor()
     p.end();
 }
 
-void DrmOutput::moveCursor(const QPoint &globalPos)
+void DrmOutput::moveCursor(Cursor* cursor, const QPoint &globalPos)
 {
-    const QMatrix4x4 hotspotMatrix = matrixDisplay(m_backend->softwareCursor().size());
+    const QMatrix4x4 hotspotMatrix = matrixDisplay(cursor->image().size());
 
-    QPoint p = globalPos - AbstractWaylandOutput::globalPos();
+    const QPoint localPos = globalPos - AbstractWaylandOutput::globalPos();
+    QPoint pos = localPos;
 
     // TODO: Do we need to handle the flipped cases differently?
     switch (transform()) {
@@ -202,22 +200,23 @@ void DrmOutput::moveCursor(const QPoint &globalPos)
         break;
     case Transform::Rotated90:
     case Transform::Flipped90:
-        p = QPoint(p.y(), pixelSize().height() - p.x());
+        pos = QPoint(localPos.y(), pixelSize().width() / scale() - localPos.x());
         break;
     case Transform::Rotated270:
     case Transform::Flipped270:
-        p = QPoint(pixelSize().width() - p.y(), p.x());
+        pos = QPoint(pixelSize().height() / scale() - localPos.y(), localPos.x());
         break;
     case Transform::Rotated180:
     case Transform::Flipped180:
-        p = QPoint(pixelSize().width() - p.x(), pixelSize().height() - p.y());
+        pos = QPoint(pixelSize().width() / scale() - localPos.x(),
+                     pixelSize().height() / scale() - localPos.y());
         break;
     default:
         Q_UNREACHABLE();
     }
-    p *= scale();
-    p -= hotspotMatrix.map(m_backend->softwareCursorHotspot());
-    drmModeMoveCursor(m_backend->fd(), m_crtc->id(), p.x(), p.y());
+    pos *= scale();
+    pos -= hotspotMatrix.map(cursor->hotspot());
+    drmModeMoveCursor(m_backend->fd(), m_crtc->id(), pos.x(), pos.y());
 }
 
 static QHash<int, QByteArray> s_connectorNames = {
@@ -276,15 +275,6 @@ bool DrmOutput::init(drmModeConnector *connector)
     setInternal(connector->connector_type == DRM_MODE_CONNECTOR_LVDS || connector->connector_type == DRM_MODE_CONNECTOR_eDP
                 || connector->connector_type == DRM_MODE_CONNECTOR_DSI);
     setDpmsSupported(true);
-
-    if (isInternal()) {
-        connect(kwinApp(), &Application::screensCreated, this,
-            [this] {
-                connect(screens()->orientationSensor(), &OrientationSensor::orientationChanged, this, &DrmOutput::automaticRotation);
-            }
-        );
-    }
-
     initOutputDevice(connector);
 
     if (!m_backend->atomicModeSetting() && !m_crtc->blank()) {
@@ -292,7 +282,7 @@ bool DrmOutput::init(drmModeConnector *connector)
         return false;
     }
 
-    updateDpms(KWayland::Server::OutputInterface::DpmsMode::On);
+    updateDpms(KWaylandServer::OutputInterface::DpmsMode::On);
     return true;
 }
 
@@ -309,11 +299,13 @@ void DrmOutput::initUuid()
 void DrmOutput::initOutputDevice(drmModeConnector *connector)
 {
     QString manufacturer;
-    if (!m_edid.eisaId().isEmpty()) {
+    if (!m_edid.vendor().isEmpty()) {
+        manufacturer = QString::fromLatin1(m_edid.vendor());
+    } else if (!m_edid.eisaId().isEmpty()) {
         manufacturer = QString::fromLatin1(m_edid.eisaId());
     }
 
-    QString connectorName = s_connectorNames.value(connector->connector_type, QByteArrayLiteral("Unknown"));
+    QString connectorName = s_connectorNames.value(connector->connector_type, QByteArrayLiteral("Unknown")) + QStringLiteral("-") + QString::number(connector->connector_type_id);
     QString modelName;
 
     if (!m_edid.monitorName().isEmpty()) {
@@ -329,23 +321,23 @@ void DrmOutput::initOutputDevice(drmModeConnector *connector)
         modelName = i18n("unknown");
     }
 
-    const QString model = connectorName + QStringLiteral("-") + QString::number(connector->connector_type_id) + QStringLiteral("-") + modelName;
+    const QString model = connectorName + QStringLiteral("-") + modelName;
 
     // read in mode information
-    QVector<KWayland::Server::OutputDeviceInterface::Mode> modes;
+    QVector<KWaylandServer::OutputDeviceInterface::Mode> modes;
     for (int i = 0; i < connector->count_modes; ++i) {
         // TODO: in AMS here we could read and store for later every mode's blob_id
         // would simplify isCurrentMode(..) and presentAtomically(..) in case of mode set
         auto *m = &connector->modes[i];
-        KWayland::Server::OutputDeviceInterface::ModeFlags deviceflags;
+        KWaylandServer::OutputDeviceInterface::ModeFlags deviceflags;
         if (isCurrentMode(m)) {
-            deviceflags |= KWayland::Server::OutputDeviceInterface::ModeFlag::Current;
+            deviceflags |= KWaylandServer::OutputDeviceInterface::ModeFlag::Current;
         }
         if (m->type & DRM_MODE_TYPE_PREFERRED) {
-            deviceflags |= KWayland::Server::OutputDeviceInterface::ModeFlag::Preferred;
+            deviceflags |= KWaylandServer::OutputDeviceInterface::ModeFlag::Preferred;
         }
 
-        KWayland::Server::OutputDeviceInterface::Mode mode;
+        KWaylandServer::OutputDeviceInterface::Mode mode;
         mode.id = i;
         mode.size = QSize(m->hdisplay, m->vdisplay);
         mode.flags = deviceflags;
@@ -365,7 +357,7 @@ void DrmOutput::initOutputDevice(drmModeConnector *connector)
         qCWarning(KWIN_DRM) << "Overwriting monitor physical size for" << m_edid.eisaId() << "/" << m_edid.monitorName() << "/" << m_edid.serialNumber() << " from " << physicalSize << "to " << overwriteSize;
         physicalSize = overwriteSize;
     }
-
+    setName(connectorName);
     initInterfaces(model, manufacturer, m_uuid, physicalSize, modes);
 }
 
@@ -387,7 +379,6 @@ bool DrmOutput::isCurrentMode(const drmModeModeInfo *mode) const
         && mode->type        == m_mode.type
         && qstrcmp(mode->name, m_mode.name) == 0;
 }
-
 void DrmOutput::initEdid(drmModeConnector *connector)
 {
     DrmScopedPointer<drmModePropertyBlobRes> edid;
@@ -544,9 +535,9 @@ void DrmOutput::atomicDisable()
     }
 }
 
-static DrmOutput::DpmsMode fromWaylandDpmsMode(KWayland::Server::OutputInterface::DpmsMode wlMode)
+static DrmOutput::DpmsMode fromWaylandDpmsMode(KWaylandServer::OutputInterface::DpmsMode wlMode)
 {
-    using namespace KWayland::Server;
+    using namespace KWaylandServer;
     switch (wlMode) {
     case OutputInterface::DpmsMode::On:
         return DrmOutput::DpmsMode::On;
@@ -561,9 +552,9 @@ static DrmOutput::DpmsMode fromWaylandDpmsMode(KWayland::Server::OutputInterface
     }
 }
 
-static KWayland::Server::OutputInterface::DpmsMode toWaylandDpmsMode(DrmOutput::DpmsMode mode)
+static KWaylandServer::OutputInterface::DpmsMode toWaylandDpmsMode(DrmOutput::DpmsMode mode)
 {
-    using namespace KWayland::Server;
+    using namespace KWaylandServer;
     switch (mode) {
     case DrmOutput::DpmsMode::On:
         return OutputInterface::DpmsMode::On;
@@ -578,7 +569,7 @@ static KWayland::Server::OutputInterface::DpmsMode toWaylandDpmsMode(DrmOutput::
     }
 }
 
-void DrmOutput::updateDpms(KWayland::Server::OutputInterface::DpmsMode mode)
+void DrmOutput::updateDpms(KWaylandServer::OutputInterface::DpmsMode mode)
 {
     if (m_dpms.isNull() || !isEnabled()) {
         return;
@@ -657,35 +648,64 @@ bool DrmOutput::dpmsLegacyApply()
     return true;
 }
 
+DrmPlane::Transformations outputToPlaneTransform(DrmOutput::Transform transform)
+ {
+    using OutTrans = DrmOutput::Transform;
+    using PlaneTrans = DrmPlane::Transformation;
+
+     // TODO: Do we want to support reflections (flips)?
+
+     switch (transform) {
+    case OutTrans::Normal:
+    case OutTrans::Flipped:
+        return PlaneTrans::Rotate0;
+    case OutTrans::Rotated90:
+    case OutTrans::Flipped90:
+        return PlaneTrans::Rotate90;
+    case OutTrans::Rotated180:
+    case OutTrans::Flipped180:
+        return PlaneTrans::Rotate180;
+    case OutTrans::Rotated270:
+    case OutTrans::Flipped270:
+        return PlaneTrans::Rotate270;
+     default:
+         Q_UNREACHABLE();
+     }
+}
+
+bool DrmOutput::hardwareTransforms() const
+{
+    if (!m_primaryPlane) {
+        return false;
+    }
+    return m_primaryPlane->transformation() == outputToPlaneTransform(transform());
+}
+
+int DrmOutput::rotation() const
+{
+    return transformToRotation(transform());
+}
+
 void DrmOutput::updateTransform(Transform transform)
 {
-    DrmPlane::Transformation planeTransform;
+    const auto planeTransform = outputToPlaneTransform(transform);
 
-    // TODO: Do we want to support reflections (flips)?
+     if (m_primaryPlane) {
+        // At the moment we have to exclude hardware transforms for vertical buffers.
+        // For that we need to support other buffers and graceful fallback from atomic tests.
+        // Reason is that standard linear buffers are not suitable.
+        const bool isPortrait = transform == Transform::Rotated90
+                                || transform == Transform::Flipped90
+                                || transform == Transform::Rotated270
+                                || transform == Transform::Flipped270;
 
-    switch (transform) {
-    case Transform::Normal:
-    case Transform::Flipped:
-        planeTransform = DrmPlane::Transformation::Rotate0;
-        break;
-    case Transform::Rotated90:
-    case Transform::Flipped90:
-        planeTransform = DrmPlane::Transformation::Rotate90;
-        break;
-    case Transform::Rotated180:
-    case Transform::Flipped180:
-        planeTransform = DrmPlane::Transformation::Rotate180;
-        break;
-    case Transform::Rotated270:
-    case Transform::Flipped270:
-        planeTransform = DrmPlane::Transformation::Rotate270;
-        break;
-    default:
-        Q_UNREACHABLE();
-    }
-
-    if (m_primaryPlane) {
-        m_primaryPlane->setTransformation(planeTransform);
+        if (!qEnvironmentVariableIsSet("KWIN_DRM_SW_ROTATIONS_ONLY") &&
+                (m_primaryPlane->supportedTransformations() & planeTransform) &&
+                !isPortrait) {
+            m_primaryPlane->setTransformation(planeTransform);
+        } else {
+            m_primaryPlane->setTransformation(DrmPlane::Transformation::Rotate0);
+        }
     }
     m_modesetRequested = true;
 
@@ -1015,12 +1035,15 @@ bool DrmOutput::doAtomicCommit(AtomicCommitMode mode)
 bool DrmOutput::atomicReqModesetPopulate(drmModeAtomicReq *req, bool enable)
 {
     if (enable) {
+        const QSize mSize = modeSize();
+        const QSize sourceSize = hardwareTransforms() ? pixelSize() : mSize;
+
         m_primaryPlane->setValue(int(DrmPlane::PropertyIndex::SrcX), 0);
         m_primaryPlane->setValue(int(DrmPlane::PropertyIndex::SrcY), 0);
-        m_primaryPlane->setValue(int(DrmPlane::PropertyIndex::SrcW), m_mode.hdisplay << 16);
-        m_primaryPlane->setValue(int(DrmPlane::PropertyIndex::SrcH), m_mode.vdisplay << 16);
-        m_primaryPlane->setValue(int(DrmPlane::PropertyIndex::CrtcW), m_mode.hdisplay);
-        m_primaryPlane->setValue(int(DrmPlane::PropertyIndex::CrtcH), m_mode.vdisplay);
+        m_primaryPlane->setValue(int(DrmPlane::PropertyIndex::SrcW), sourceSize.width() << 16);
+        m_primaryPlane->setValue(int(DrmPlane::PropertyIndex::SrcH), sourceSize.height() << 16);
+        m_primaryPlane->setValue(int(DrmPlane::PropertyIndex::CrtcW), mSize.width());
+        m_primaryPlane->setValue(int(DrmPlane::PropertyIndex::CrtcH), mSize.height());
         m_primaryPlane->setValue(int(DrmPlane::PropertyIndex::CrtcId), m_crtc->id());
     } else {
         if (m_backend->deleteBufferAfterPageFlip()) {
@@ -1060,47 +1083,6 @@ bool DrmOutput::supportsTransformations() const
         || transformations.testFlag(DrmPlane::Transformation::Rotate270);
 }
 
-void DrmOutput::automaticRotation()
-{
-    if (!m_primaryPlane) {
-        return;
-    }
-    const auto supportedTransformations = m_primaryPlane->supportedTransformations();
-    const auto requestedTransformation = screens()->orientationSensor()->orientation();
-
-    Transform newTransformation = Transform::Normal;
-    switch (requestedTransformation) {
-    case OrientationSensor::Orientation::TopUp:
-        newTransformation = Transform::Normal;
-        break;
-    case OrientationSensor::Orientation::TopDown:
-        if (!supportedTransformations.testFlag(DrmPlane::Transformation::Rotate180)) {
-            return;
-        }
-        newTransformation = Transform::Rotated180;
-        break;
-    case OrientationSensor::Orientation::LeftUp:
-        if (!supportedTransformations.testFlag(DrmPlane::Transformation::Rotate90)) {
-            return;
-        }
-        newTransformation = Transform::Rotated90;
-        break;
-    case OrientationSensor::Orientation::RightUp:
-        if (!supportedTransformations.testFlag(DrmPlane::Transformation::Rotate270)) {
-            return;
-        }
-        newTransformation = Transform::Rotated270;
-        break;
-    case OrientationSensor::Orientation::FaceUp:
-    case OrientationSensor::Orientation::FaceDown:
-    case OrientationSensor::Orientation::Undefined:
-        // unsupported
-        return;
-    }
-    setTransform(newTransformation);
-    emit screens()->changed();
-}
-
 int DrmOutput::gammaRampSize() const
 {
     return m_crtc->gammaRampSize();
@@ -1112,3 +1094,10 @@ bool DrmOutput::setGammaRamp(const GammaRamp &gamma)
 }
 
 }
+
+QDebug& operator<<(QDebug& s, const KWin::DrmOutput* output)
+{
+    if (!output)
+        return s.nospace() << "DrmOutput()";
+    return s.nospace() << "DrmOutput(" << output->name() << ", crtc:" << output->crtc() << ", connector:" << output->connector() << ", geometry:" << output->geometry() << ')';
+}
