@@ -49,9 +49,9 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #include "decorations/decoratedclient.h"
 #include <logging.h>
 
-#include <KWayland/Server/buffer_interface.h>
-#include <KWayland/Server/subcompositor_interface.h>
-#include <KWayland/Server/surface_interface.h>
+#include <KWaylandServer/buffer_interface.h>
+#include <KWaylandServer/subcompositor_interface.h>
+#include <KWaylandServer/surface_interface.h>
 
 #include <array>
 #include <cmath>
@@ -567,10 +567,12 @@ void SceneOpenGL::insertWait()
  */
 void SceneOpenGL2::paintCursor()
 {
+    Cursor* cursor = Cursors::self()->currentCursor();
+
     // don't paint if we use hardware cursor or the cursor is hidden
     if (!kwinApp()->platform()->usesSoftwareCursor() ||
         kwinApp()->platform()->isCursorHidden() ||
-        kwinApp()->platform()->softwareCursor().isNull()) {
+        cursor->image().isNull()) {
         return;
     }
 
@@ -578,7 +580,7 @@ void SceneOpenGL2::paintCursor()
     if (!m_cursorTexture) {
         auto updateCursorTexture = [this] {
             // don't paint if no image for cursor is set
-            const QImage img = kwinApp()->platform()->softwareCursor();
+            const QImage img = Cursors::self()->currentCursor()->image();
             if (img.isNull()) {
                 return;
             }
@@ -589,11 +591,11 @@ void SceneOpenGL2::paintCursor()
         updateCursorTexture();
 
         // handle shape update on case cursor image changed
-        connect(kwinApp()->platform(), &Platform::cursorChanged, this, updateCursorTexture);
+        connect(Cursors::self(), &Cursors::currentCursorChanged, this, updateCursorTexture);
     }
 
     // get cursor position in projection coordinates
-    const QPoint cursorPos = Cursor::pos() - kwinApp()->platform()->softwareCursorHotspot();
+    const QPoint cursorPos = cursor->pos() - cursor->hotspot();
     const QRect cursorRect(0, 0, m_cursorTexture->width(), m_cursorTexture->height());
     QMatrix4x4 mvp = m_projectionMatrix;
     mvp.translate(cursorPos.x(), cursorPos.y());
@@ -609,12 +611,12 @@ void SceneOpenGL2::paintCursor()
     m_cursorTexture->render(QRegion(cursorRect), cursorRect);
     m_cursorTexture->unbind();
 
-    kwinApp()->platform()->markCursorAsRendered();
+    cursor->markAsRendered();
 
     glDisable(GL_BLEND);
 }
 
-qint64 SceneOpenGL::paint(QRegion damage, QList<Toplevel *> toplevels)
+qint64 SceneOpenGL::paint(const QRegion &damage, const QList<Toplevel *> &toplevels)
 {
     // actually paint the frame, flushed with the NEXT frame
     createStackingOrder(toplevels);
@@ -647,7 +649,7 @@ qint64 SceneOpenGL::paint(QRegion damage, QList<Toplevel *> toplevels)
 
             int mask = 0;
             updateProjectionMatrix();
-            paintScreen(&mask, damage.intersected(geo), repaint, &update, &valid, projectionMatrix(), geo);   // call generic implementation
+            paintScreen(&mask, damage.intersected(geo), repaint, &update, &valid, projectionMatrix(), geo, screens()->scale(i));   // call generic implementation
             paintCursor();
 
             GLVertexBuffer::streamingBuffer()->endOfFrame();
@@ -734,7 +736,7 @@ QMatrix4x4 SceneOpenGL::transformation(int mask, const ScreenPaintData &data) co
     return matrix;
 }
 
-void SceneOpenGL::paintBackground(QRegion region)
+void SceneOpenGL::paintBackground(const QRegion &region)
 {
     PaintClipper pc(region);
     if (!PaintClipper::clip()) {
@@ -1001,14 +1003,14 @@ void SceneOpenGL2::updateProjectionMatrix()
     m_projectionMatrix = createProjectionMatrix();
 }
 
-void SceneOpenGL2::paintSimpleScreen(int mask, QRegion region)
+void SceneOpenGL2::paintSimpleScreen(int mask, const QRegion &region)
 {
     m_screenProjectionMatrix = m_projectionMatrix;
 
     Scene::paintSimpleScreen(mask, region);
 }
 
-void SceneOpenGL2::paintGenericScreen(int mask, ScreenPaintData data)
+void SceneOpenGL2::paintGenericScreen(int mask, const ScreenPaintData &data)
 {
     const QMatrix4x4 screenMatrix = transformation(mask, data);
 
@@ -1035,7 +1037,7 @@ Scene::Window *SceneOpenGL2::createWindow(Toplevel *t)
     return new OpenGLWindow(t, this);
 }
 
-void SceneOpenGL2::finalDrawWindow(EffectWindowImpl* w, int mask, QRegion region, WindowPaintData& data)
+void SceneOpenGL2::finalDrawWindow(EffectWindowImpl* w, int mask, const QRegion &region, WindowPaintData& data)
 {
     if (waylandServer() && waylandServer()->isScreenLocked() && !w->window()->isLockScreen() && !w->window()->isInputMethod()) {
         return;
@@ -1043,7 +1045,7 @@ void SceneOpenGL2::finalDrawWindow(EffectWindowImpl* w, int mask, QRegion region
     performPaintWindow(w, mask, region, data);
 }
 
-void SceneOpenGL2::performPaintWindow(EffectWindowImpl* w, int mask, QRegion region, WindowPaintData& data)
+void SceneOpenGL2::performPaintWindow(EffectWindowImpl* w, int mask, const QRegion &region, WindowPaintData& data)
 {
     if (mask & PAINT_WINDOW_LANCZOS) {
         if (!m_lanczosFilter) {
@@ -1075,16 +1077,13 @@ OpenGLWindow::~OpenGLWindow()
 {
 }
 
-static SceneOpenGLTexture *s_frameTexture = nullptr;
 // Bind the window pixmap to an OpenGL texture.
 bool OpenGLWindow::bindTexture()
 {
-    s_frameTexture = nullptr;
     OpenGLWindowPixmap *pixmap = windowPixmap<OpenGLWindowPixmap>();
     if (!pixmap) {
         return false;
     }
-    s_frameTexture = pixmap->texture();
     if (pixmap->isDiscarded()) {
         return !pixmap->texture()->isNull();
     }
@@ -1153,7 +1152,7 @@ bool OpenGLWindow::beginRenderWindow(int mask, const QRegion &region, WindowPain
     if (data.quads.isEmpty())
         return false;
 
-    if (!bindTexture() || !s_frameTexture) {
+    if (!bindTexture()) {
         return false;
     }
 
@@ -1161,20 +1160,6 @@ bool OpenGLWindow::beginRenderWindow(int mask, const QRegion &region, WindowPain
         glEnable(GL_SCISSOR_TEST);
     }
 
-    // Update the texture filter
-    if (waylandServer()) {
-        filter = Scene::ImageFilterGood;
-        s_frameTexture->setFilter(GL_LINEAR);
-    } else {
-        if (options->glSmoothScale() != 0 &&
-            (mask & (Scene::PAINT_WINDOW_TRANSFORMED | Scene::PAINT_SCREEN_TRANSFORMED)))
-            filter = Scene::ImageFilterGood;
-        else
-            filter = Scene::ImageFilterFast;
-
-        s_frameTexture->setFilter(filter == Scene::ImageFilterGood ? GL_LINEAR : GL_NEAREST);
-    }
-
     const GLVertexAttrib attribs[] = {
         { VA_Position, 2, GL_FLOAT, offsetof(GLVertex2D, position) },
         { VA_TexCoord, 2, GL_FLOAT, offsetof(GLVertex2D, texcoord) },
@@ -1243,40 +1228,151 @@ void OpenGLWindow::setBlendEnabled(bool enabled)
     m_blendingEnabled = enabled;
 }
 
-void OpenGLWindow::setupLeafNodes(LeafNode *nodes, const WindowQuadList *quads, const WindowPaintData &data)
+/**
+ * \internal
+ *
+ * Counts the total number of pixmaps in the tree with the given root \a windowPixmap.
+ */
+static int windowPixmapCount(WindowPixmap *windowPixmap)
 {
-    if (!quads[ShadowLeaf].isEmpty()) {
-        nodes[ShadowLeaf].texture = static_cast<SceneOpenGLShadow *>(m_shadow)->shadowTexture();
-        nodes[ShadowLeaf].opacity = data.opacity();
-        nodes[ShadowLeaf].hasAlpha = true;
-        nodes[ShadowLeaf].coordinateType = NormalizedCoordinates;
+    int count = 1; // 1 for the window pixmap itself.
+
+    const QVector<WindowPixmap *> children = windowPixmap->children();
+    for (WindowPixmap *child : children)
+        count += windowPixmapCount(child);
+
+    return count;
+}
+
+void OpenGLWindow::initializeRenderContext(RenderContext &context, const WindowPaintData &data)
+{
+    WindowPixmap *currentPixmap = windowPixmap<OpenGLWindowPixmap>();
+
+    context.shadowOffset = 0;
+    context.decorationOffset = 1;
+    context.contentOffset = 2;
+    context.previousContentOffset = windowPixmapCount(currentPixmap) + 2;
+    context.quadCount = data.quads.count();
+
+    const int nodeCount = context.previousContentOffset + 1;
+
+    QVector<RenderNode> &renderNodes = context.renderNodes;
+    renderNodes.resize(nodeCount);
+
+    for (const WindowQuad &quad : data.quads) {
+        switch (quad.type()) {
+        case WindowQuadShadow:
+            renderNodes[context.shadowOffset].quads << quad;
+            break;
+
+        case WindowQuadDecoration:
+            renderNodes[context.decorationOffset].quads << quad;
+            break;
+
+        case WindowQuadContents:
+            renderNodes[context.contentOffset + quad.id()].quads << quad;
+            break;
+
+        default:
+            // Ignore window quad generated by effects.
+            break;
+        }
     }
 
-    if (!quads[DecorationLeaf].isEmpty()) {
-        nodes[DecorationLeaf].texture = getDecorationTexture();
-        nodes[DecorationLeaf].opacity = data.opacity();
-        nodes[DecorationLeaf].hasAlpha = true;
-        nodes[DecorationLeaf].coordinateType = UnnormalizedCoordinates;
+    RenderNode &shadowRenderNode = renderNodes[context.shadowOffset];
+    if (!shadowRenderNode.quads.isEmpty()) {
+        SceneOpenGLShadow *shadow = static_cast<SceneOpenGLShadow *>(m_shadow);
+        shadowRenderNode.texture = shadow->shadowTexture();
+        shadowRenderNode.opacity = data.opacity();
+        shadowRenderNode.hasAlpha = true;
+        shadowRenderNode.coordinateType = NormalizedCoordinates;
+        shadowRenderNode.leafType = ShadowLeaf;
     }
 
-    nodes[ContentLeaf].texture = s_frameTexture;
-    nodes[ContentLeaf].hasAlpha = !isOpaque();
-    // TODO: ARGB crsoofading is atm. a hack, playing on opacities for two dumb SrcOver operations
-    // Should be a shader
+    RenderNode &decorationRenderNode = renderNodes[context.decorationOffset];
+    if (!decorationRenderNode.quads.isEmpty()) {
+        decorationRenderNode.texture = getDecorationTexture();
+        decorationRenderNode.opacity = data.opacity();
+        decorationRenderNode.hasAlpha = true;
+        decorationRenderNode.coordinateType = UnnormalizedCoordinates;
+        decorationRenderNode.leafType = DecorationLeaf;
+    }
+
+    // FIXME: Cross-fading must be implemented in a shader.
+    float contentOpacity = data.opacity();
     if (data.crossFadeProgress() != 1.0 && (data.opacity() < 0.95 || toplevel->hasAlpha())) {
         const float opacity = 1.0 - data.crossFadeProgress();
-        nodes[ContentLeaf].opacity = data.opacity() * (1 - pow(opacity, 1.0f + 2.0f * data.opacity()));
-    } else {
-        nodes[ContentLeaf].opacity = data.opacity();
+        contentOpacity *= 1 - pow(opacity, 1.0f + 2.0f * data.opacity());
     }
-    nodes[ContentLeaf].coordinateType = UnnormalizedCoordinates;
 
+    // The main surface and all of its sub-surfaces form a tree. In order to initialize
+    // the render nodes for the window pixmaps we need to traverse the tree in the
+    // depth-first search manner. The id of content window quads corresponds to the time
+    // when we visited the corresponding window pixmap. The DFS traversal probably doesn't
+    // have a significant impact on performance. However, if that's the case, we could
+    // keep a cache of window pixmaps in the order in which they'll be rendered.
+    QStack<WindowPixmap *> stack;
+    stack.push(currentPixmap);
+
+    int i = 0;
+
+    while (!stack.isEmpty()) {
+        OpenGLWindowPixmap *windowPixmap = static_cast<OpenGLWindowPixmap *>(stack.pop());
+
+        // If it's an unmapped sub-surface, don't render it and all of its children.
+        if (!windowPixmap->isValid())
+            continue;
+
+        RenderNode &contentRenderNode = renderNodes[context.contentOffset + i++];
+        contentRenderNode.texture = windowPixmap->texture();
+        contentRenderNode.hasAlpha = windowPixmap->hasAlphaChannel();
+        contentRenderNode.opacity = contentOpacity;
+        contentRenderNode.coordinateType = UnnormalizedCoordinates;
+        contentRenderNode.leafType = ContentLeaf;
+
+        const QVector<WindowPixmap *> children = windowPixmap->children();
+        for (WindowPixmap *child : children)
+            stack.push(child);
+    }
+
+    // Note that cross-fading is currently working properly only on X11. In order to make it
+    // work on Wayland, we have to render the current and the previous window pixmap trees in
+    // offscreen render targets, then use a cross-fading shader to blend those two layers.
     if (data.crossFadeProgress() != 1.0) {
         OpenGLWindowPixmap *previous = previousWindowPixmap<OpenGLWindowPixmap>();
-        nodes[PreviousContentLeaf].texture = previous ? previous->texture() : nullptr;
-        nodes[PreviousContentLeaf].hasAlpha = !isOpaque();
-        nodes[PreviousContentLeaf].opacity = data.opacity() * (1.0 - data.crossFadeProgress());
-        nodes[PreviousContentLeaf].coordinateType = NormalizedCoordinates;
+        if (previous) { // TODO(vlad): Should cross-fading be disabled on Wayland?
+            const QRect &oldGeometry = previous->contentsRect();
+            RenderNode &previousContentRenderNode = renderNodes[context.previousContentOffset];
+            for (const WindowQuad &quad : qAsConst(renderNodes[context.contentOffset].quads)) {
+                // We need to create new window quads with normalized texture coordinates.
+                // Normal quads divide the x/y position by width/height. This would not work
+                // as the texture is larger than the visible content in case of a decorated
+                // Client resulting in garbage being shown. So we calculate the normalized
+                // texture coordinate in the Client's new content space and map it to the
+                // previous Client's content space.
+                WindowQuad newQuad(WindowQuadContents);
+                for (int i = 0; i < 4; ++i) {
+                    const qreal xFactor = (quad[i].textureX() - toplevel->clientPos().x())
+                            / qreal(toplevel->clientSize().width());
+                    const qreal yFactor = (quad[i].textureY() - toplevel->clientPos().y())
+                            / qreal(toplevel->clientSize().height());
+                    const qreal u = (xFactor * oldGeometry.width() + oldGeometry.x())
+                            / qreal(previous->size().width());
+                    const qreal v = (yFactor * oldGeometry.height() + oldGeometry.y())
+                            / qreal(previous->size().height());
+                    newQuad[i] = WindowVertex(quad[i].x(), quad[i].y(), u, v);
+                }
+                previousContentRenderNode.quads.append(newQuad);
+            }
+
+            previousContentRenderNode.texture = previous->texture();
+            previousContentRenderNode.hasAlpha = previous->hasAlphaChannel();
+            previousContentRenderNode.opacity = data.opacity() * (1.0 - data.crossFadeProgress());
+            previousContentRenderNode.coordinateType = NormalizedCoordinates;
+            previousContentRenderNode.leafType = PreviousContentLeaf;
+
+            context.quadCount += previousContentRenderNode.quads.count();
+        }
     }
 }
 
@@ -1304,37 +1400,9 @@ QMatrix4x4 OpenGLWindow::modelViewProjectionMatrix(int mask, const WindowPaintDa
     return scene->projectionMatrix() * mvMatrix;
 }
 
-void OpenGLWindow::renderSubSurface(GLShader *shader, const QMatrix4x4 &mvp, const QMatrix4x4 &windowMatrix, OpenGLWindowPixmap *pixmap, const QRegion &region, bool hardwareClipping)
-{
-    QMatrix4x4 newWindowMatrix = windowMatrix;
-    newWindowMatrix.translate(pixmap->subSurface()->position().x(), pixmap->subSurface()->position().y());
-
-    qreal scale = 1.0;
-    if (pixmap->surface()) {
-        scale = pixmap->surface()->scale();
-    }
-
-    if (!pixmap->texture()->isNull()) {
-        setBlendEnabled(pixmap->buffer() && pixmap->buffer()->hasAlphaChannel());
-        // render this texture
-        shader->setUniform(GLShader::ModelViewProjectionMatrix, mvp * newWindowMatrix);
-        auto texture = pixmap->texture();
-        texture->bind();
-        texture->render(region, QRect(0, 0, texture->width() / scale, texture->height() / scale), hardwareClipping);
-        texture->unbind();
-    }
-
-    const auto &children = pixmap->children();
-    for (auto pixmap : children) {
-        if (pixmap->subSurface().isNull() || pixmap->subSurface()->surface().isNull() || !pixmap->subSurface()->surface()->isMapped()) {
-            continue;
-        }
-        renderSubSurface(shader, mvp, newWindowMatrix, static_cast<OpenGLWindowPixmap*>(pixmap), region, hardwareClipping);
-    }
-}
-
-void OpenGLWindow::performPaint(int mask, QRegion region, WindowPaintData data)
+void OpenGLWindow::performPaint(int mask, const QRegion &region, const WindowPaintData &_data)
 {
+    WindowPaintData data = _data;
     if (!beginRenderWindow(mask, region, data))
         return;
 
@@ -1342,7 +1410,6 @@ void OpenGLWindow::performPaint(int mask, QRegion region, WindowPaintData data)
     const QMatrix4x4 modelViewProjection = modelViewProjectionMatrix(mask, data);
     const QMatrix4x4 mvpMatrix = modelViewProjection * windowMatrix;
 
-
     bool useX11TextureClamp = false;
 
     GLShader *shader = data.shader;
@@ -1379,76 +1446,30 @@ void OpenGLWindow::performPaint(int mask, QRegion region, WindowPaintData data)
 
     shader->setUniform(GLShader::Saturation, data.saturation());
 
-    WindowQuadList quads[LeafCount];
-
-    // Split the quads into separate lists for each type
-    foreach (const WindowQuad &quad, data.quads) {
-        switch (quad.type()) {
-        case WindowQuadDecoration:
-            quads[DecorationLeaf].append(quad);
-            continue;
-
-        case WindowQuadContents:
-            quads[ContentLeaf].append(quad);
-            continue;
-
-        case WindowQuadShadow:
-            quads[ShadowLeaf].append(quad);
-            continue;
-
-        default:
-            continue;
-        }
-    }
-
-    if (data.crossFadeProgress() != 1.0) {
-        OpenGLWindowPixmap *previous = previousWindowPixmap<OpenGLWindowPixmap>();
-        if (previous) {
-            const QRect &oldGeometry = previous->contentsRect();
-            for (const WindowQuad &quad : quads[ContentLeaf]) {
-                // we need to create new window quads with normalize texture coordinates
-                // normal quads divide the x/y position by width/height. This would not work as the texture
-                // is larger than the visible content in case of a decorated Client resulting in garbage being shown.
-                // So we calculate the normalized texture coordinate in the Client's new content space and map it to
-                // the previous Client's content space.
-                WindowQuad newQuad(WindowQuadContents);
-                for (int i = 0; i < 4; ++i) {
-                    const qreal xFactor = qreal(quad[i].textureX() - toplevel->clientPos().x())/qreal(toplevel->clientSize().width());
-                    const qreal yFactor = qreal(quad[i].textureY() - toplevel->clientPos().y())/qreal(toplevel->clientSize().height());
-                    WindowVertex vertex(quad[i].x(), quad[i].y(),
-                                        (xFactor * oldGeometry.width() + oldGeometry.x())/qreal(previous->size().width()),
-                                        (yFactor * oldGeometry.height() + oldGeometry.y())/qreal(previous->size().height()));
-                    newQuad[i] = vertex;
-                }
-                quads[PreviousContentLeaf].append(newQuad);
-            }
-        }
-    }
+    RenderContext renderContext;
+    initializeRenderContext(renderContext, data);
 
     const bool indexedQuads = GLVertexBuffer::supportsIndexedQuads();
     const GLenum primitiveType = indexedQuads ? GL_QUADS : GL_TRIANGLES;
     const int verticesPerQuad = indexedQuads ? 4 : 6;
 
-    const size_t size = verticesPerQuad *
-        (quads[0].count() + quads[1].count() + quads[2].count() + quads[3].count()) * sizeof(GLVertex2D);
+    const size_t size = verticesPerQuad * renderContext.quadCount * sizeof(GLVertex2D);
 
     GLVertexBuffer *vbo = GLVertexBuffer::streamingBuffer();
     GLVertex2D *map = (GLVertex2D *) vbo->map(size);
 
-    LeafNode nodes[LeafCount];
-    setupLeafNodes(nodes, quads, data);
-
-    for (int i = 0, v = 0; i < LeafCount; i++) {
-        if (quads[i].isEmpty() || !nodes[i].texture)
+    for (int i = 0, v = 0; i < renderContext.renderNodes.count(); i++) {
+        RenderNode &renderNode = renderContext.renderNodes[i];
+        if (renderNode.quads.isEmpty() || !renderNode.texture)
             continue;
 
-        nodes[i].firstVertex = v;
-        nodes[i].vertexCount = quads[i].count() * verticesPerQuad;
+        renderNode.firstVertex = v;
+        renderNode.vertexCount = renderNode.quads.count() * verticesPerQuad;
 
-        const QMatrix4x4 matrix = nodes[i].texture->matrix(nodes[i].coordinateType);
+        const QMatrix4x4 matrix = renderNode.texture->matrix(renderNode.coordinateType);
 
-        quads[i].makeInterleavedArrays(primitiveType, &map[v], matrix);
-        v += quads[i].count() * verticesPerQuad;
+        renderNode.quads.makeInterleavedArrays(primitiveType, &map[v], matrix);
+        v += renderNode.quads.count() * verticesPerQuad;
     }
 
     vbo->unmap();
@@ -1459,23 +1480,24 @@ void OpenGLWindow::performPaint(int mask, QRegion region, WindowPaintData data)
 
     float opacity = -1.0;
 
-    for (int i = 0; i < LeafCount; i++) {
-        if (nodes[i].vertexCount == 0)
+    for (int i = 0; i < renderContext.renderNodes.count(); i++) {
+        const RenderNode &renderNode = renderContext.renderNodes[i];
+        if (renderNode.vertexCount == 0)
             continue;
 
-        setBlendEnabled(nodes[i].hasAlpha || nodes[i].opacity < 1.0);
+        setBlendEnabled(renderNode.hasAlpha || renderNode.opacity < 1.0);
 
-        if (opacity != nodes[i].opacity) {
+        if (opacity != renderNode.opacity) {
             shader->setUniform(GLShader::ModulationConstant,
-                               modulate(nodes[i].opacity, data.brightness()));
-            opacity = nodes[i].opacity;
+                               modulate(renderNode.opacity, data.brightness()));
+            opacity = renderNode.opacity;
         }
 
-        nodes[i].texture->setFilter(filter);
-        nodes[i].texture->setWrapMode(GL_CLAMP_TO_EDGE);
-        nodes[i].texture->bind();
+        renderNode.texture->setFilter(filter);
+        renderNode.texture->setWrapMode(GL_CLAMP_TO_EDGE);
+        renderNode.texture->bind();
 
-        if (i == ContentLeaf && useX11TextureClamp) {
+        if (renderNode.leafType == ContentLeaf && useX11TextureClamp) {
             // X11 windows are reparented to have their buffer in the middle of a larger texture
             // holding the frame window.
             // This code passes the texture geometry to the fragment shader
@@ -1494,23 +1516,12 @@ void OpenGLWindow::performPaint(int mask, QRegion region, WindowPaintData data)
             shader->setUniform(GLShader::TextureClamp, QVector4D({0, 0, 1, 1}));
         }
 
-        vbo->draw(region, primitiveType, nodes[i].firstVertex, nodes[i].vertexCount, m_hardwareClipping);
+        vbo->draw(region, primitiveType, renderNode.firstVertex,
+                  renderNode.vertexCount, m_hardwareClipping);
     }
 
     vbo->unbindArrays();
 
-    // render sub-surfaces
-    auto wp = windowPixmap<OpenGLWindowPixmap>();
-    const auto &children = wp ? wp->children() : QVector<WindowPixmap*>();
-    const QPoint mainSurfaceOffset = bufferOffset();
-    windowMatrix.translate(mainSurfaceOffset.x(), mainSurfaceOffset.y());
-    for (auto pixmap : children) {
-        if (pixmap->subSurface().isNull() || pixmap->subSurface()->surface().isNull() || !pixmap->subSurface()->surface()->isMapped()) {
-            continue;
-        }
-        renderSubSurface(shader, modelViewProjection, windowMatrix, static_cast<OpenGLWindowPixmap*>(pixmap), region, m_hardwareClipping);
-    }
-
     setBlendEnabled(false);
 
     if (!data.shader)
@@ -1531,7 +1542,7 @@ OpenGLWindowPixmap::OpenGLWindowPixmap(Scene::Window *window, SceneOpenGL* scene
 {
 }
 
-OpenGLWindowPixmap::OpenGLWindowPixmap(const QPointer<KWayland::Server::SubSurfaceInterface> &subSurface, WindowPixmap *parent, SceneOpenGL *scene)
+OpenGLWindowPixmap::OpenGLWindowPixmap(const QPointer<KWaylandServer::SubSurfaceInterface> &subSurface, WindowPixmap *parent, SceneOpenGL *scene)
     : WindowPixmap(subSurface, parent)
     , m_texture(scene->createTexture())
     , m_scene(scene)
@@ -1566,10 +1577,6 @@ static bool needsPixmapUpdate(const OpenGLWindowPixmap *pixmap)
 bool OpenGLWindowPixmap::bind()
 {
     if (!m_texture->isNull()) {
-        // always call updateBuffer to get the sub-surface tree updated
-        if (subSurface().isNull() && !toplevel()->damage().isEmpty()) {
-            updateBuffer();
-        }
         if (needsPixmapUpdate(this)) {
             m_texture->updateFromPixmap(this);
             // mipmaps need to be updated
@@ -1584,11 +1591,6 @@ bool OpenGLWindowPixmap::bind()
         }
         return true;
     }
-    // also bind all children, needs to be done before checking isValid
-    // as there might be valid children to render, see https://bugreports.qt.io/browse/QTBUG-52192
-    if (subSurface().isNull()) {
-        updateBuffer();
-    }
     for (auto it = children().constBegin(); it != children().constEnd(); ++it) {
         static_cast<OpenGLWindowPixmap*>(*it)->bind();
     }
@@ -1607,7 +1609,7 @@ bool OpenGLWindowPixmap::bind()
     return success;
 }
 
-WindowPixmap *OpenGLWindowPixmap::createChild(const QPointer<KWayland::Server::SubSurfaceInterface> &subSurface)
+WindowPixmap *OpenGLWindowPixmap::createChild(const QPointer<KWaylandServer::SubSurfaceInterface> &subSurface)
 {
     return new OpenGLWindowPixmap(subSurface, this, m_scene);
 }
@@ -1711,12 +1713,13 @@ void SceneOpenGL::EffectFrame::crossFadeText()
     m_textTexture = nullptr;
 }
 
-void SceneOpenGL::EffectFrame::render(QRegion region, double opacity, double frameOpacity)
+void SceneOpenGL::EffectFrame::render(const QRegion &_region, double opacity, double frameOpacity)
 {
     if (m_effectFrame->geometry().isEmpty())
         return; // Nothing to display
 
-    region = infiniteRegion(); // TODO: Old region doesn't seem to work with OpenGL
+    Q_UNUSED(_region);
+    const QRegion region = infiniteRegion(); // TODO: Old region doesn't seem to work with OpenGL
 
     GLShader* shader = m_effectFrame->shader();
     if (!shader) {
@@ -2109,7 +2112,7 @@ QSharedPointer<GLTexture> DecorationShadowTextureCache::getTexture(SceneOpenGLSh
 {
     Q_ASSERT(shadow->hasDecorationShadow());
     unregister(shadow);
-    const auto &decoShadow = shadow->decorationShadow();
+    const auto &decoShadow = shadow->decorationShadow().toStrongRef();
     Q_ASSERT(!decoShadow.isNull());
     auto it = m_cache.find(decoShadow.data());
     if (it != m_cache.end()) {
