@@ -77,13 +77,14 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #include "overlaywindow.h"
 #include "screens.h"
 #include "shadow.h"
+#include "subsurfacemonitor.h"
 #include "wayland_server.h"
 
 #include "thumbnailitem.h"
 
-#include <KWayland/Server/buffer_interface.h>
-#include <KWayland/Server/subcompositor_interface.h>
-#include <KWayland/Server/surface_interface.h>
+#include <KWaylandServer/buffer_interface.h>
+#include <KWaylandServer/subcompositor_interface.h>
+#include <KWaylandServer/surface_interface.h>
 
 namespace KWin
 {
@@ -105,7 +106,7 @@ Scene::~Scene()
 
 // returns mask and possibly modified region
 void Scene::paintScreen(int* mask, const QRegion &damage, const QRegion &repaint,
-                        QRegion *updateRegion, QRegion *validRegion, const QMatrix4x4 &projection, const QRect &outputGeometry)
+                        QRegion *updateRegion, QRegion *validRegion, const QMatrix4x4 &projection, const QRect &outputGeometry, const qreal screenScale)
 {
     const QSize &screenSize = screens()->size();
     const QRegion displayRegion(0, 0, screenSize.width(), screenSize.height());
@@ -145,7 +146,7 @@ void Scene::paintScreen(int* mask, const QRegion &damage, const QRegion &repaint
         paintBackground(region);
     }
 
-    ScreenPaintData data(projection, outputGeometry);
+    ScreenPaintData data(projection, outputGeometry, screenScale);
     effects->paintScreen(*mask, region, data);
 
     foreach (Window *w, stacking_order) {
@@ -191,7 +192,7 @@ void Scene::idle()
 }
 
 // the function that'll be eventually called by paintScreen() above
-void Scene::finalPaintScreen(int mask, QRegion region, ScreenPaintData& data)
+void Scene::finalPaintScreen(int mask, const QRegion &region, ScreenPaintData& data)
 {
     if (mask & (PAINT_SCREEN_TRANSFORMED | PAINT_SCREEN_WITH_TRANSFORMED_WINDOWS))
         paintGenericScreen(mask, data);
@@ -201,7 +202,7 @@ void Scene::finalPaintScreen(int mask, QRegion region, ScreenPaintData& data)
 
 // The generic painting code that can handle even transformations.
 // It simply paints bottom-to-top.
-void Scene::paintGenericScreen(int orig_mask, ScreenPaintData)
+void Scene::paintGenericScreen(int orig_mask, const ScreenPaintData &)
 {
     if (!(orig_mask & PAINT_SCREEN_BACKGROUND_FIRST)) {
         paintBackground(infiniteRegion());
@@ -211,6 +212,9 @@ void Scene::paintGenericScreen(int orig_mask, ScreenPaintData)
     foreach (Window * w, stacking_order) { // bottom to top
         Toplevel* topw = w->window();
 
+        // Let the scene window update the window pixmap tree.
+        w->preprocess();
+
         // Reset the repaint_region.
         // This has to be done here because many effects schedule a repaint for
         // the next frame within Effects::prePaintWindow.
@@ -246,7 +250,7 @@ void Scene::paintGenericScreen(int orig_mask, ScreenPaintData)
 // The optimized case without any transformations at all.
 // It can paint only the requested region and can use clipping
 // to reduce painting and improve performance.
-void Scene::paintSimpleScreen(int orig_mask, QRegion region)
+void Scene::paintSimpleScreen(int orig_mask, const QRegion &region)
 {
     Q_ASSERT((orig_mask & (PAINT_SCREEN_TRANSFORMED
                          | PAINT_SCREEN_WITH_TRANSFORMED_WINDOWS)) == 0);
@@ -266,6 +270,9 @@ void Scene::paintSimpleScreen(int orig_mask, QRegion region)
         data.paint = region;
         data.paint |= toplevel->repaints();
 
+        // Let the scene window update the window pixmap tree.
+        window->preprocess();
+
         // Reset the repaint_region.
         // This has to be done here because many effects schedule a repaint for
         // the next frame within Effects::prePaintWindow.
@@ -370,7 +377,7 @@ void Scene::paintSimpleScreen(int orig_mask, QRegion region)
 
     if (fullRepaint) {
         painted_region = displayRegion;
-        damaged_region = displayRegion;
+        damaged_region = displayRegion - repaintClip;
     } else {
         painted_region |= paintedArea;
 
@@ -389,25 +396,40 @@ void Scene::addToplevel(Toplevel *c)
     Q_ASSERT(!m_windows.contains(c));
     Scene::Window *w = createWindow(c);
     m_windows[ c ] = w;
+
+    auto discardPixmap = [w]() { w->discardPixmap(); };
+    auto discardQuads = [w]() { w->invalidateQuadsCache(); };
+
     connect(c, SIGNAL(geometryShapeChanged(KWin::Toplevel*,QRect)), SLOT(windowGeometryShapeChanged(KWin::Toplevel*)));
     connect(c, SIGNAL(windowClosed(KWin::Toplevel*,KWin::Deleted*)), SLOT(windowClosed(KWin::Toplevel*,KWin::Deleted*)));
-    //A change of scale won't affect the geometry in compositor co-ordinates, but will affect the window quads.
     if (c->surface()) {
-        connect(c->surface(), &KWayland::Server::SurfaceInterface::scaleChanged, this, std::bind(&Scene::windowGeometryShapeChanged, this, c));
+        // We generate window quads for sub-surfaces so it's quite important to discard
+        // the pixmap tree and cached window quads when the sub-surface tree is changed.
+        SubSurfaceMonitor *monitor = new SubSurfaceMonitor(c->surface(), this);
+
+        // TODO(vlad): Is there a more efficient way to manage window pixmap trees?
+        connect(monitor, &SubSurfaceMonitor::subSurfaceAdded, this, discardPixmap);
+        connect(monitor, &SubSurfaceMonitor::subSurfaceRemoved, this, discardPixmap);
+        connect(monitor, &SubSurfaceMonitor::subSurfaceResized, this, discardPixmap);
+        connect(monitor, &SubSurfaceMonitor::subSurfaceMapped, this, discardPixmap);
+        connect(monitor, &SubSurfaceMonitor::subSurfaceUnmapped, this, discardPixmap);
+
+        connect(monitor, &SubSurfaceMonitor::subSurfaceAdded, this, discardQuads);
+        connect(monitor, &SubSurfaceMonitor::subSurfaceRemoved, this, discardQuads);
+        connect(monitor, &SubSurfaceMonitor::subSurfaceMoved, this, discardQuads);
+        connect(monitor, &SubSurfaceMonitor::subSurfaceResized, this, discardQuads);
+        connect(monitor, &SubSurfaceMonitor::subSurfaceMapped, this, discardQuads);
+        connect(monitor, &SubSurfaceMonitor::subSurfaceUnmapped, this, discardQuads);
+
+        connect(c->surface(), &KWaylandServer::SurfaceInterface::scaleChanged, this, discardQuads);
     }
-    connect(c, &Toplevel::screenScaleChanged, this,
-        [this, c] {
-            windowGeometryShapeChanged(c);
-        }
-    );
+
+    connect(c, &Toplevel::screenScaleChanged, this, discardQuads);
+    connect(c, &Toplevel::shadowChanged, this, discardQuads);
+
     c->effectWindow()->setSceneWindow(w);
     c->updateShadow();
     w->updateShadow(c->shadow());
-    connect(c, &Toplevel::shadowChanged, this,
-        [w] {
-            w->invalidateQuadsCache();
-        }
-    );
 }
 
 void Scene::removeToplevel(Toplevel *toplevel)
@@ -441,7 +463,7 @@ void Scene::windowGeometryShapeChanged(Toplevel *c)
     w->discardShape();
 }
 
-void Scene::createStackingOrder(QList<Toplevel *> toplevels)
+void Scene::createStackingOrder(const QList<Toplevel *> &toplevels)
 {
     // TODO: cache the stacking_order in case it has not changed
     foreach (Toplevel *c, toplevels) {
@@ -457,11 +479,10 @@ void Scene::clearStackingOrder()
 
 static Scene::Window *s_recursionCheck = nullptr;
 
-void Scene::paintWindow(Window* w, int mask, QRegion region, WindowQuadList quads)
+void Scene::paintWindow(Window* w, int mask, const QRegion &_region, const WindowQuadList &quads)
 {
     // no painting outside visible screen (and no transformations)
-    const QSize &screenSize = screens()->size();
-    region &= QRect(0, 0, screenSize.width(), screenSize.height());
+    const QRegion region = _region & QRect({0, 0}, screens()->size());
     if (region.isEmpty())  // completely clipped
         return;
     if (w->window()->isDeleted() && w->window()->skipsCloseAnimation()) {
@@ -504,7 +525,7 @@ static void adjustClipRegion(AbstractThumbnailItem *item, QRegion &clippingRegio
     }
 }
 
-void Scene::paintWindowThumbnails(Scene::Window *w, QRegion region, qreal opacity, qreal brightness, qreal saturation)
+void Scene::paintWindowThumbnails(Scene::Window *w, const QRegion &region, qreal opacity, qreal brightness, qreal saturation)
 {
     EffectWindowImpl *wImpl = static_cast<EffectWindowImpl*>(effectWindow(w));
     for (QHash<WindowThumbnailItem*, QPointer<EffectWindowImpl> >::const_iterator it = wImpl->thumbnails().constBegin();
@@ -601,13 +622,13 @@ void Scene::paintDesktop(int desktop, int mask, const QRegion &region, ScreenPai
 }
 
 // the function that'll be eventually called by paintWindow() above
-void Scene::finalPaintWindow(EffectWindowImpl* w, int mask, QRegion region, WindowPaintData& data)
+void Scene::finalPaintWindow(EffectWindowImpl* w, int mask, const QRegion &region, WindowPaintData& data)
 {
     effects->drawWindow(w, mask, region, data);
 }
 
 // will be eventually called from drawWindow()
-void Scene::finalDrawWindow(EffectWindowImpl* w, int mask, QRegion region, WindowPaintData& data)
+void Scene::finalDrawWindow(EffectWindowImpl* w, int mask, const QRegion &region, WindowPaintData& data)
 {
     if (waylandServer() && waylandServer()->isScreenLocked() && !w->window()->isLockScreen() && !w->window()->isInputMethod()) {
         return;
@@ -733,7 +754,9 @@ void Scene::Window::updatePixmap()
     if (m_currentPixmap.isNull()) {
         m_currentPixmap.reset(createWindowPixmap());
     }
-    if (!m_currentPixmap->isValid()) {
+    if (m_currentPixmap->isValid()) {
+        m_currentPixmap->update();
+    } else {
         m_currentPixmap->create();
     }
 }
@@ -780,11 +803,8 @@ QRegion Scene::Window::bufferShape() const
 
 QRegion Scene::Window::clientShape() const
 {
-    if (AbstractClient *client = qobject_cast<AbstractClient *>(toplevel)) {
-        if (client->isShade()) {
-            return QRegion();
-        }
-    }
+    if (isShaded())
+        return QRegion();
 
     const QRegion shape = bufferShape();
     const QMargins bufferMargins = toplevel->bufferMargins();
@@ -826,6 +846,13 @@ bool Scene::Window::isOpaque() const
     return toplevel->opacity() == 1.0 && !toplevel->hasAlpha();
 }
 
+bool Scene::Window::isShaded() const
+{
+    if (AbstractClient *client = qobject_cast<AbstractClient *>(toplevel))
+        return client->isShade();
+    return false;
+}
+
 bool Scene::Window::isPaintingEnabled() const
 {
     return !disable_painting;
@@ -870,7 +897,11 @@ WindowQuadList Scene::Window::buildQuads(bool force) const
     if (cached_quad_list != nullptr && !force)
         return *cached_quad_list;
 
-    WindowQuadList ret = makeContentsQuads();
+    WindowQuadList ret;
+
+    if (!isShaded()) {
+        ret += makeContentsQuads();
+    }
 
     if (!toplevel->frameMargins().isNull()) {
         AbstractClient *client = dynamic_cast<AbstractClient*>(toplevel);
@@ -970,36 +1001,66 @@ WindowQuadList Scene::Window::makeDecorationQuads(const QRect *rects, const QReg
 
 WindowQuadList Scene::Window::makeContentsQuads() const
 {
-    const QRegion contentsRegion = clientShape();
-    if (contentsRegion.isEmpty()) {
-        return WindowQuadList();
-    }
+    // TODO(vlad): What about the case where we need to build window quads for a deleted
+    // window? Presumably, the current window will be invalid so no window quads will be
+    // generated. Is it okay?
 
-    const QPointF geometryOffset = bufferOffset();
-    const qreal textureScale = toplevel->bufferScale();
+    WindowPixmap *currentPixmap = windowPixmap<WindowPixmap>();
+    if (!currentPixmap)
+        return WindowQuadList();
 
     WindowQuadList quads;
-    quads.reserve(contentsRegion.rectCount());
+    int id = 0;
 
-    for (const QRectF &rect : contentsRegion) {
-        WindowQuad quad(WindowQuadContents);
+    // We need to assign an id to each generated window quad in order to be able to match
+    // a list of window quads against a particular window pixmap. We traverse the window
+    // pixmap tree in the depth-first search manner and assign an id to each window quad.
+    // The id is the time when we visited the window pixmap.
 
-        const qreal x0 = rect.left() + geometryOffset.x();
-        const qreal y0 = rect.top() + geometryOffset.y();
-        const qreal x1 = rect.right() + geometryOffset.x();
-        const qreal y1 = rect.bottom() + geometryOffset.y();
+    QStack<WindowPixmap *> stack;
+    stack.push(currentPixmap);
 
-        const qreal u0 = rect.left() * textureScale;
-        const qreal v0 = rect.top() * textureScale;
-        const qreal u1 = rect.right() * textureScale;
-        const qreal v1 = rect.bottom() * textureScale;
+    while (!stack.isEmpty()) {
+        WindowPixmap *windowPixmap = stack.pop();
 
-        quad[0] = WindowVertex(QPointF(x0, y0), QPointF(u0, v0));
-        quad[1] = WindowVertex(QPointF(x1, y0), QPointF(u1, v0));
-        quad[2] = WindowVertex(QPointF(x1, y1), QPointF(u1, v1));
-        quad[3] = WindowVertex(QPointF(x0, y1), QPointF(u0, v1));
+        // If it's an unmapped sub-surface, don't generate window quads for it.
+        if (!windowPixmap->isValid())
+            continue;
 
-        quads << quad;
+        const QRegion region = windowPixmap->shape();
+        const QPoint position = windowPixmap->framePosition();
+        const qreal scale = windowPixmap->scale();
+        const int quadId = id++;
+
+        for (const QRect &rect : region) {
+            // Note that the window quad id is not unique if the window is shaped, i.e. the
+            // region contains more than just one rectangle. We assume that the "source" quad
+            // had been subdivided.
+            WindowQuad quad(WindowQuadContents, quadId);
+
+            const qreal x0 = rect.x() + position.x();
+            const qreal y0 = rect.y() + position.y();
+            const qreal x1 = rect.x() + rect.width() + position.x();
+            const qreal y1 = rect.y() + rect.height() + position.y();
+
+            const qreal u0 = rect.x() * scale;
+            const qreal v0 = rect.y() * scale;
+            const qreal u1 = (rect.x() + rect.width()) * scale;
+            const qreal v1 = (rect.y() + rect.height()) * scale;
+
+            quad[0] = WindowVertex(QPointF(x0, y0), QPointF(u0, v0));
+            quad[1] = WindowVertex(QPointF(x1, y0), QPointF(u1, v0));
+            quad[2] = WindowVertex(QPointF(x1, y1), QPointF(u1, v1));
+            quad[3] = WindowVertex(QPointF(x0, y1), QPointF(u0, v1));
+
+            quads << quad;
+        }
+
+        // Push the child window pixmaps onto the stack, remember we're visiting the pixmaps
+        // in the depth-first search manner.
+        const auto children = windowPixmap->children();
+        for (WindowPixmap *child : children)
+            stack.push(child);
     }
 
     return quads;
@@ -1019,6 +1080,16 @@ void Scene::Window::updateShadow(Shadow* shadow)
     m_shadow = shadow;
 }
 
+void Scene::Window::preprocess()
+{
+    // The tracked damage will be reset after the scene is done with copying buffer's data.
+    // Note that we have to be prepared for the case where no damage has occurred since kwin
+    // core may discard the current window pixmap at any moment.
+    if (!m_currentPixmap || !window()->damage().isEmpty()) {
+        updatePixmap();
+    }
+}
+
 //****************************************
 // WindowPixmap
 //****************************************
@@ -1029,7 +1100,7 @@ WindowPixmap::WindowPixmap(Scene::Window *window)
 {
 }
 
-WindowPixmap::WindowPixmap(const QPointer<KWayland::Server::SubSurfaceInterface> &subSurface, WindowPixmap *parent)
+WindowPixmap::WindowPixmap(const QPointer<KWaylandServer::SubSurfaceInterface> &subSurface, WindowPixmap *parent)
     : m_window(parent->m_window)
     , m_pixmap(XCB_PIXMAP_NONE)
     , m_discarded(false)
@@ -1046,7 +1117,7 @@ WindowPixmap::~WindowPixmap()
         xcb_free_pixmap(connection(), m_pixmap);
     }
     if (m_buffer) {
-        using namespace KWayland::Server;
+        using namespace KWaylandServer;
         QObject::disconnect(m_buffer.data(), &BufferInterface::aboutToBeDestroyed, m_buffer.data(), &BufferInterface::unref);
         m_buffer->unref();
     }
@@ -1060,8 +1131,8 @@ void WindowPixmap::create()
     // always update from Buffer on Wayland, don't try using XPixmap
     if (kwinApp()->shouldUseWaylandForCompositing()) {
         // use Buffer
-        updateBuffer();
-        if ((m_buffer || !m_fbo.isNull()) && m_subSurface.isNull()) {
+        update();
+        if (!isRoot() && isValid()) {
             m_window->unreferencePreviousPixmap();
         }
         return;
@@ -1095,27 +1166,13 @@ void WindowPixmap::create()
     m_window->unreferencePreviousPixmap();
 }
 
-WindowPixmap *WindowPixmap::createChild(const QPointer<KWayland::Server::SubSurfaceInterface> &subSurface)
+void WindowPixmap::update()
 {
-    Q_UNUSED(subSurface)
-    return nullptr;
-}
-
-bool WindowPixmap::isValid() const
-{
-    if (!m_buffer.isNull() || !m_fbo.isNull() || !m_internalImage.isNull()) {
-        return true;
-    }
-    return m_pixmap != XCB_PIXMAP_NONE;
-}
-
-void WindowPixmap::updateBuffer()
-{
-    using namespace KWayland::Server;
+    using namespace KWaylandServer;
     if (SurfaceInterface *s = surface()) {
         QVector<WindowPixmap*> oldTree = m_children;
         QVector<WindowPixmap*> children;
-        using namespace KWayland::Server;
+        using namespace KWaylandServer;
         const auto subSurfaces = s->childSubSurfaces();
         for (const auto &subSurface : subSurfaces) {
             if (subSurface.isNull()) {
@@ -1124,7 +1181,7 @@ void WindowPixmap::updateBuffer()
             auto it = std::find_if(oldTree.begin(), oldTree.end(), [subSurface] (WindowPixmap *p) { return p->m_subSurface == subSurface; });
             if (it != oldTree.end()) {
                 children << *it;
-                (*it)->updateBuffer();
+                (*it)->update();
                 oldTree.erase(it);
             } else {
                 WindowPixmap *p = createChild(subSurface);
@@ -1168,7 +1225,26 @@ void WindowPixmap::updateBuffer()
     }
 }
 
-KWayland::Server::SurfaceInterface *WindowPixmap::surface() const
+WindowPixmap *WindowPixmap::createChild(const QPointer<KWaylandServer::SubSurfaceInterface> &subSurface)
+{
+    Q_UNUSED(subSurface)
+    return nullptr;
+}
+
+bool WindowPixmap::isValid() const
+{
+    if (!m_buffer.isNull() || !m_fbo.isNull() || !m_internalImage.isNull()) {
+        return true;
+    }
+    return m_pixmap != XCB_PIXMAP_NONE;
+}
+
+bool WindowPixmap::isRoot() const
+{
+    return !m_parent;
+}
+
+KWaylandServer::SurfaceInterface *WindowPixmap::surface() const
 {
     if (!m_subSurface.isNull()) {
         return m_subSurface->surface().data();
@@ -1177,6 +1253,39 @@ KWayland::Server::SurfaceInterface *WindowPixmap::surface() const
     }
 }
 
+QPoint WindowPixmap::position() const
+{
+    if (subSurface())
+        return subSurface()->position();
+    return m_window->bufferOffset();
+}
+
+QPoint WindowPixmap::framePosition() const
+{
+    return position() + (m_parent ? m_parent->framePosition() : QPoint());
+}
+
+qreal WindowPixmap::scale() const
+{
+    if (surface())
+        return surface()->scale();
+    return toplevel()->bufferScale();
+}
+
+QRegion WindowPixmap::shape() const
+{
+    if (subSurface())
+        return QRect(QPoint(), surface()->size());
+    return m_window->clientShape();
+}
+
+bool WindowPixmap::hasAlphaChannel() const
+{
+    if (buffer())
+        return buffer()->hasAlphaChannel();
+    return toplevel()->hasAlpha();
+}
+
 //****************************************
 // Scene::EffectFrame
 //****************************************
