@@ -36,20 +36,18 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
 #include <KDecoration2/DecoratedClient>
 #include <KDecoration2/Decoration>
-#include <KWayland/Server/appmenu_interface.h>
-#include <KWayland/Server/buffer_interface.h>
-#include <KWayland/Server/clientconnection.h>
-#include <KWayland/Server/display.h>
-#include <KWayland/Server/plasmashell_interface.h>
-#include <KWayland/Server/plasmawindowmanagement_interface.h>
-#include <KWayland/Server/qtsurfaceextension_interface.h>
-#include <KWayland/Server/seat_interface.h>
-#include <KWayland/Server/server_decoration_interface.h>
-#include <KWayland/Server/server_decoration_palette_interface.h>
-#include <KWayland/Server/shadow_interface.h>
-#include <KWayland/Server/subcompositor_interface.h>
-#include <KWayland/Server/surface_interface.h>
-#include <KWayland/Server/xdgdecoration_interface.h>
+#include <KWaylandServer/appmenu_interface.h>
+#include <KWaylandServer/buffer_interface.h>
+#include <KWaylandServer/clientconnection.h>
+#include <KWaylandServer/display.h>
+#include <KWaylandServer/plasmashell_interface.h>
+#include <KWaylandServer/plasmawindowmanagement_interface.h>
+#include <KWaylandServer/seat_interface.h>
+#include <KWaylandServer/server_decoration_interface.h>
+#include <KWaylandServer/server_decoration_palette_interface.h>
+#include <KWaylandServer/shadow_interface.h>
+#include <KWaylandServer/surface_interface.h>
+#include <KWaylandServer/xdgdecoration_interface.h>
 
 #include <QFileInfo>
 
@@ -60,7 +58,7 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
 Q_DECLARE_METATYPE(NET::WindowType)
 
-using namespace KWayland::Server;
+using namespace KWaylandServer;
 
 namespace KWin
 {
@@ -153,7 +151,7 @@ void XdgShellClient::init()
         connect(this, &AbstractClient::clientStartUserMovedResized, this, configure);
         connect(this, &AbstractClient::clientFinishUserMovedResized, this, configure);
 
-        connect(this, &XdgShellClient::geometryChanged, this, &XdgShellClient::updateClientOutputs);
+        connect(this, &XdgShellClient::frameGeometryChanged, this, &XdgShellClient::updateClientOutputs);
         connect(screens(), &Screens::changed, this, &XdgShellClient::updateClientOutputs);
     } else if (m_xdgShellPopup) {
         connect(m_xdgShellPopup, &XdgShellPopupInterface::configureAcknowledged, this, &XdgShellClient::handleConfigureAcknowledged);
@@ -169,7 +167,7 @@ void XdgShellClient::init()
     updateShadow();
     connect(surface(), &SurfaceInterface::shadowChanged, this, &Toplevel::updateShadow);
 
-    connect(waylandServer(), &WaylandServer::foreignTransientChanged, this, [this](KWayland::Server::SurfaceInterface *child) {
+    connect(waylandServer(), &WaylandServer::foreignTransientChanged, this, [this](KWaylandServer::SurfaceInterface *child) {
         if (child == surface()) {
             handleTransientForChanged();
         }
@@ -192,7 +190,7 @@ void XdgShellClient::finishInit()
     if (supportsWindowRules()) {
         setupWindowRules(false);
 
-        const QRect originalGeometry = QRect(pos(), sizeForClientSize(clientSize()));
+        const QRect originalGeometry = frameGeometry();
         const QRect ruledGeometry = rules()->checkGeometry(originalGeometry, true);
         if (originalGeometry != ruledGeometry) {
             setFrameGeometry(ruledGeometry);
@@ -320,31 +318,28 @@ QPoint XdgShellClient::clientContentPos() const
     return -1 * clientPos();
 }
 
-static QRect subSurfaceTreeRect(const SurfaceInterface *surface, const QPoint &position = QPoint())
-{
-    QRect rect(position, surface->size());
-
-    const QList<QPointer<SubSurfaceInterface>> subSurfaces = surface->childSubSurfaces();
-    for (const QPointer<SubSurfaceInterface> &subSurface : subSurfaces) {
-        if (Q_UNLIKELY(!subSurface)) {
-            continue;
-        }
-        const SurfaceInterface *child = subSurface->surface();
-        if (Q_UNLIKELY(!child)) {
-            continue;
-        }
-        rect |= subSurfaceTreeRect(child, position + subSurface->position());
-    }
-
-    return rect;
-}
-
 QSize XdgShellClient::clientSize() const
 {
-    const QRect boundingRect = subSurfaceTreeRect(surface());
+    const QRect boundingRect = surface()->boundingRect();
     return m_windowGeometry.size().boundedTo(boundingRect.size());
 }
 
+QSize XdgShellClient::minSize() const
+{
+    if (m_xdgShellToplevel) {
+        return rules()->checkMinSize(m_xdgShellToplevel->minimumSize());
+    }
+    return QSize(0, 0);
+}
+
+QSize XdgShellClient::maxSize() const
+{
+    if (m_xdgShellToplevel) {
+        return rules()->checkMaxSize(m_xdgShellToplevel->maximumSize());
+    }
+    return QSize(INT_MAX, INT_MAX);
+}
+
 void XdgShellClient::debug(QDebug &stream) const
 {
     stream.nospace();
@@ -357,7 +352,7 @@ bool XdgShellClient::belongsToDesktop() const
     const auto clients = waylandServer()->clients();
 
     return std::any_of(clients.constBegin(), clients.constEnd(),
-        [this](const XdgShellClient *client) {
+        [this](const AbstractClient *client) {
             if (belongsToSameApplication(client, SameApplicationChecks())) {
                 return client->isDesktop();
             }
@@ -444,31 +439,6 @@ void XdgShellClient::markAsMapped()
     updateShowOnScreenEdge();
 }
 
-void XdgShellClient::createDecoration(const QRect &oldGeom)
-{
-    KDecoration2::Decoration *decoration = Decoration::DecorationBridge::self()->createDecoration(this);
-    if (decoration) {
-        QMetaObject::invokeMethod(decoration, "update", Qt::QueuedConnection);
-        connect(decoration, &KDecoration2::Decoration::shadowChanged, this, &Toplevel::updateShadow);
-        connect(decoration, &KDecoration2::Decoration::bordersChanged, this,
-            [this]() {
-                GeometryUpdatesBlocker blocker(this);
-                RequestGeometryBlocker requestBlocker(this);
-                const QRect oldGeometry = frameGeometry();
-                if (!isShade()) {
-                    checkWorkspacePosition(oldGeometry);
-                }
-                emit geometryShapeChanged(this, oldGeometry);
-            }
-        );
-    }
-    setDecoration(decoration);
-    // TODO: ensure the new geometry still fits into the client area (e.g. maximized windows)
-    doSetGeometry(QRect(oldGeom.topLeft(), m_windowGeometry.size() + QSize(borderLeft() + borderRight(), borderBottom() + borderTop())));
-
-    emit geometryShapeChanged(this, oldGeom);
-}
-
 void XdgShellClient::updateDecoration(bool check_workspace_pos, bool force)
 {
     if (!force &&
@@ -484,7 +454,7 @@ void XdgShellClient::updateDecoration(bool check_workspace_pos, bool force)
     } else
         destroyDecoration();
     if (m_serverDecoration && isDecorated()) {
-        m_serverDecoration->setMode(KWayland::Server::ServerSideDecorationManagerInterface::Mode::Server);
+        m_serverDecoration->setMode(KWaylandServer::ServerSideDecorationManagerInterface::Mode::Server);
     }
     if (m_xdgDecoration) {
         auto mode = isDecorated() || m_userNoBorder ? XdgDecorationInterface::Mode::ServerSide: XdgDecorationInterface::Mode::ClientSide;
@@ -499,9 +469,9 @@ void XdgShellClient::updateDecoration(bool check_workspace_pos, bool force)
     blockGeometryUpdates(false);
 }
 
-void XdgShellClient::setFrameGeometry(int x, int y, int w, int h, ForceGeometry_t force)
+void XdgShellClient::setFrameGeometry(const QRect &rect, ForceGeometry_t force)
 {
-    const QRect newGeometry = rules()->checkGeometry(QRect(x, y, w, h));
+    const QRect newGeometry = rules()->checkGeometry(rect);
 
     if (areGeometryUpdatesBlocked()) {
         // when the GeometryUpdateBlocker exits the current geom is passed to setGeometry
@@ -569,9 +539,9 @@ void XdgShellClient::doSetGeometry(const QRect &rect)
         return;
     }
 
-    if (m_unmapped && m_geomMaximizeRestore.isEmpty() && !m_frameGeometry.isEmpty()) {
+    if (m_unmapped && geometryRestore().isEmpty() && !m_frameGeometry.isEmpty()) {
         // use first valid geometry as restore geometry
-        m_geomMaximizeRestore = m_frameGeometry;
+        setGeometryRestore(m_frameGeometry);
     }
 
     if (frameGeometryIsChanged) {
@@ -579,12 +549,13 @@ void XdgShellClient::doSetGeometry(const QRect &rect)
             workspace()->updateClientArea();
         }
         updateWindowRules(Rules::Position | Rules::Size);
+        emit frameGeometryChanged(this, frameGeometryBeforeUpdateBlocking());
     }
 
-    const auto old = frameGeometryBeforeUpdateBlocking();
+    emit geometryShapeChanged(this, frameGeometryBeforeUpdateBlocking());
+
     addRepaintDuringGeometryUpdates();
     updateGeometryBeforeUpdateBlocking();
-    emit geometryShapeChanged(this, old);
 
     if (isResize()) {
         performMoveResize();
@@ -783,7 +754,7 @@ void XdgShellClient::changeMaximize(bool horizontal, bool vertical, bool adjust)
     }
 
     const QRect clientArea = isElectricBorderMaximizing() ?
-        workspace()->clientArea(MaximizeArea, Cursor::pos(), desktop()) :
+        workspace()->clientArea(MaximizeArea, Cursors::self()->mouse()->pos(), desktop()) :
         workspace()->clientArea(MaximizeArea, this);
 
     const MaximizeMode oldMode = m_requestedMaximizeMode;
@@ -808,7 +779,7 @@ void XdgShellClient::changeMaximize(bool horizontal, bool vertical, bool adjust)
     // call into decoration update borders
     if (isDecorated() && decoration()->client() && !(options->borderlessMaximizedWindows() && m_requestedMaximizeMode == KWin::MaximizeFull)) {
         changeMaximizeRecursion = true;
-        const auto c = decoration()->client().data();
+        const auto c = decoration()->client().toStrongRef();
         if ((m_requestedMaximizeMode & MaximizeVertical) != (oldMode & MaximizeVertical)) {
             emit c->maximizedVerticallyChanged(m_requestedMaximizeMode & MaximizeVertical);
         }
@@ -833,7 +804,7 @@ void XdgShellClient::changeMaximize(bool horizontal, bool vertical, bool adjust)
     const auto oldQuickTileMode = quickTileMode();
     if (quickTileMode() != QuickTileMode(QuickTileFlag::None)) {
         if (oldMode == MaximizeFull &&
-                !clientArea.contains(m_geomMaximizeRestore.center())) {
+                !clientArea.contains(geometryRestore().center())) {
             // Not restoring on the same screen
             // TODO: The following doesn't work for some reason
             //quick_tile_mode = QuickTileNone; // And exit quick tile mode manually
@@ -845,7 +816,7 @@ void XdgShellClient::changeMaximize(bool horizontal, bool vertical, bool adjust)
     }
 
     if (m_requestedMaximizeMode == MaximizeFull) {
-        m_geomMaximizeRestore = oldGeometry;
+        setGeometryRestore(oldGeometry);
         // TODO: Client has more checks
         if (options->electricBorderMaximize()) {
             updateQuickTileMode(QuickTileFlag::Maximize);
@@ -865,19 +836,14 @@ void XdgShellClient::changeMaximize(bool horizontal, bool vertical, bool adjust)
             emit quickTileModeChanged();
         }
 
-        if (m_geomMaximizeRestore.isValid()) {
-            setFrameGeometry(m_geomMaximizeRestore);
+        if (geometryRestore().isValid()) {
+            setFrameGeometry(geometryRestore());
         } else {
             setFrameGeometry(workspace()->clientArea(PlacementArea, this));
         }
     }
 }
 
-void XdgShellClient::setGeometryRestore(const QRect &geo)
-{
-    m_geomMaximizeRestore = geo;
-}
-
 MaximizeMode XdgShellClient::maximizeMode() const
 {
     return m_maximizeMode;
@@ -888,11 +854,6 @@ MaximizeMode XdgShellClient::requestedMaximizeMode() const
     return m_requestedMaximizeMode;
 }
 
-QRect XdgShellClient::geometryRestore() const
-{
-    return m_geomMaximizeRestore;
-}
-
 bool XdgShellClient::noBorder() const
 {
     if (m_serverDecoration) {
@@ -930,7 +891,7 @@ void XdgShellClient::setFullScreen(bool set, bool user)
     }
 
     if (wasFullscreen) {
-        workspace()->updateFocusMousePosition(Cursor::pos()); // may cause leave event
+        workspace()->updateFocusMousePosition(Cursors::self()->mouse()->pos()); // may cause leave event
     } else {
         m_geomFsRestore = frameGeometry();
     }
@@ -951,7 +912,7 @@ void XdgShellClient::setFullScreen(bool set, bool user)
     } else {
         if (m_geomFsRestore.isValid()) {
             int currentScreen = screen();
-            setFrameGeometry(QRect(m_geomFsRestore.topLeft(), adjustedSize(m_geomFsRestore.size())));
+            setFrameGeometry(QRect(m_geomFsRestore.topLeft(), constrainFrameSize(m_geomFsRestore.size())));
             if( currentScreen != screen())
                 workspace()->sendClientToScreen( this, currentScreen );
         } else {
@@ -984,7 +945,7 @@ void XdgShellClient::setOnAllActivities(bool set)
     Q_UNUSED(set)
 }
 
-bool XdgShellClient::takeFocus()
+void XdgShellClient::takeFocus()
 {
     if (rules()->checkAcceptFocus(wantsInput())) {
         if (m_xdgShellToplevel) {
@@ -996,8 +957,6 @@ bool XdgShellClient::takeFocus()
     if (!keepAbove() && !isOnScreenDisplay() && !belongsToDesktop()) {
         workspace()->setShowingDesktop(false);
     }
-
-    return true;
 }
 
 void XdgShellClient::doSetActive()
@@ -1175,7 +1134,7 @@ void XdgShellClient::handleTransientForChanged()
     if (!transientSurface) {
         transientSurface = waylandServer()->findForeignTransientForSurface(surface());
     }
-    XdgShellClient *transientClient = waylandServer()->findClient(transientSurface);
+    AbstractClient *transientClient = waylandServer()->findClient(transientSurface);
     if (transientClient != transientFor()) {
         // Remove from main client.
         if (transientFor()) {
@@ -1221,7 +1180,7 @@ void XdgShellClient::handleMoveRequested(SeatInterface *seat, quint32 serial)
     // FIXME: Check the seat and serial.
     Q_UNUSED(seat)
     Q_UNUSED(serial)
-    performMouseCommand(Options::MouseMove, Cursor::pos());
+    performMouseCommand(Options::MouseMove, Cursors::self()->mouse()->pos());
 }
 
 void XdgShellClient::handleResizeRequested(SeatInterface *seat, quint32 serial, Qt::Edges edges)
@@ -1236,7 +1195,7 @@ void XdgShellClient::handleResizeRequested(SeatInterface *seat, quint32 serial,
         finishMoveResize(false);
     }
     setMoveResizePointerButtonDown(true);
-    setMoveOffset(Cursor::pos() - pos());  // map from global
+    setMoveOffset(Cursors::self()->mouse()->pos() - pos());  // map from global
     setInvertedMoveOffset(rect().bottomRight() - moveOffset());
     setUnrestrictedMoveResize(false);
     auto toPosition = [edges] {
@@ -1262,7 +1221,7 @@ void XdgShellClient::handleResizeRequested(SeatInterface *seat, quint32 serial,
 
 void XdgShellClient::handleMinimizeRequested()
 {
-    performMouseCommand(Options::MouseMinimize, Cursor::pos());
+    performMouseCommand(Options::MouseMinimize, Cursors::self()->mouse()->pos());
 }
 
 void XdgShellClient::handleMaximizeRequested(bool maximized)
@@ -1341,7 +1300,7 @@ void XdgShellClient::handleCommitted()
     }
 
     if (!m_hasWindowGeometry) {
-        m_windowGeometry = subSurfaceTreeRect(surface());
+        m_windowGeometry = surface()->boundingRect();
     }
 
     updatePendingGeometry();
@@ -1350,17 +1309,11 @@ void XdgShellClient::handleCommitted()
     markAsMapped();
 }
 
-void XdgShellClient::resizeWithChecks(int w, int h, ForceGeometry_t force)
+void XdgShellClient::resizeWithChecks(const QSize &size, ForceGeometry_t force)
 {
-    const QRect area = workspace()->clientArea(WorkArea, this);
     // don't allow growing larger than workarea
-    if (w > area.width()) {
-        w = area.width();
-    }
-    if (h > area.height()) {
-        h = area.height();
-    }
-    setFrameGeometry(x(), y(), w, h, force);
+    const QRect area = workspace()->clientArea(WorkArea, this);
+    setFrameGeometry(QRect{pos(), size.boundedTo(area.size())}, force);
 }
 
 void XdgShellClient::unmap()
@@ -1420,6 +1373,11 @@ void XdgShellClient::installPlasmaShellSurface(PlasmaShellSurfaceInterface *surf
             workspace()->updateClientArea();
         }
     };
+    connect(surface, &PlasmaShellSurfaceInterface::panelTakesFocusChanged , this, [this, surface]() {
+        if (surface->panelTakesFocus()) {
+            workspace()->activateClient(this);
+        }
+    });
     connect(surface, &PlasmaShellSurfaceInterface::positionChanged, this, updatePosition);
     connect(surface, &PlasmaShellSurfaceInterface::roleChanged, this, updateRole);
     connect(surface, &PlasmaShellSurfaceInterface::panelBehaviorChanged, this,
@@ -1442,10 +1400,11 @@ void XdgShellClient::installPlasmaShellSurface(PlasmaShellSurfaceInterface *surf
             m_plasmaShellSurface->showAutoHidingPanel();
         }
     );
-    updatePosition();
+    if (surface->isPositionSet())
+        updatePosition();
     updateRole();
     updateShowOnScreenEdge();
-    connect(this, &XdgShellClient::geometryChanged, this, &XdgShellClient::updateShowOnScreenEdge);
+    connect(this, &XdgShellClient::frameGeometryChanged, this, &XdgShellClient::updateShowOnScreenEdge);
 
     setSkipTaskbar(surface->skipTaskbar());
     connect(surface, &PlasmaShellSurfaceInterface::skipTaskbarChanged, this, [this] {
@@ -1634,6 +1593,8 @@ bool XdgShellClient::hasTransientPlacementHint() const
 
 QRect XdgShellClient::transientPlacement(const QRect &bounds) const
 {
+    Q_ASSERT(m_xdgShellPopup);
+
     QRect anchorRect;
     Qt::Edges anchorEdge;
     Qt::Edges gravity;
@@ -1642,7 +1603,6 @@ QRect XdgShellClient::transientPlacement(const QRect &bounds) const
     QSize size = frameGeometry().size();
 
     const QPoint parentClientPos = transientFor()->pos() + transientFor()->clientPos();
-    QRect popupPosition;
 
     // returns if a target is within the supplied bounds, optional edges argument states which side to check
     auto inBounds = [bounds](const QRect &target, Qt::Edges edges = Qt::LeftEdge | Qt::RightEdge | Qt::TopEdge | Qt::BottomEdge) -> bool {
@@ -1662,31 +1622,25 @@ QRect XdgShellClient::transientPlacement(const QRect &bounds) const
         return true;
     };
 
-    if (m_xdgShellPopup) {
-        anchorRect = m_xdgShellPopup->anchorRect();
-        anchorEdge = m_xdgShellPopup->anchorEdge();
-        gravity = m_xdgShellPopup->gravity();
-        offset = m_xdgShellPopup->anchorOffset();
-        constraintAdjustments = m_xdgShellPopup->constraintAdjustments();
-        if (!size.isValid()) {
-            size = m_xdgShellPopup->initialSize();
-        }
-    } else {
-        Q_UNREACHABLE();
+    anchorRect = m_xdgShellPopup->anchorRect();
+    anchorEdge = m_xdgShellPopup->anchorEdge();
+    gravity = m_xdgShellPopup->gravity();
+    offset = m_xdgShellPopup->anchorOffset();
+    constraintAdjustments = m_xdgShellPopup->constraintAdjustments();
+    if (!size.isValid()) {
+        size = m_xdgShellPopup->initialSize();
     }
 
-
-    //initial position
-    popupPosition = QRect(popupOffset(anchorRect, anchorEdge, gravity, size) + offset + parentClientPos, size);
+    QRect popupRect(popupOffset(anchorRect, anchorEdge, gravity, size) + offset + parentClientPos, size);
 
     //if that fits, we don't need to do anything
-    if (inBounds(popupPosition)) {
-        return popupPosition;
+    if (inBounds(popupRect)) {
+        return popupRect;
     }
     //otherwise apply constraint adjustment per axis in order XDG Shell Popup states
 
     if (constraintAdjustments & PositionerConstraint::FlipX) {
-        if (!inBounds(popupPosition, Qt::LeftEdge | Qt::RightEdge)) {
+        if (!inBounds(popupRect, Qt::LeftEdge | Qt::RightEdge)) {
             //flip both edges (if either bit is set, XOR both)
             auto flippedAnchorEdge = anchorEdge;
             if (flippedAnchorEdge & (Qt::LeftEdge | Qt::RightEdge)) {
@@ -1696,30 +1650,39 @@ QRect XdgShellClient::transientPlacement(const QRect &bounds) const
             if (flippedGravity & (Qt::LeftEdge | Qt::RightEdge)) {
                 flippedGravity ^= (Qt::LeftEdge | Qt::RightEdge);
             }
-            auto flippedPopupPosition = QRect(popupOffset(anchorRect, flippedAnchorEdge, flippedGravity, size) + offset + parentClientPos, size);
+            auto flippedPopupRect = QRect(popupOffset(anchorRect, flippedAnchorEdge, flippedGravity, size) + offset + parentClientPos, size);
 
             //if it still doesn't fit we should continue with the unflipped version
-            if (inBounds(flippedPopupPosition, Qt::LeftEdge | Qt::RightEdge)) {
-                popupPosition.moveLeft(flippedPopupPosition.x());
+            if (inBounds(flippedPopupRect, Qt::LeftEdge | Qt::RightEdge)) {
+                popupRect.moveLeft(flippedPopupRect.left());
             }
         }
     }
     if (constraintAdjustments & PositionerConstraint::SlideX) {
-        if (!inBounds(popupPosition, Qt::LeftEdge)) {
-            popupPosition.moveLeft(bounds.x());
+        if (!inBounds(popupRect, Qt::LeftEdge)) {
+            popupRect.moveLeft(bounds.left());
         }
-        if (!inBounds(popupPosition, Qt::RightEdge)) {
-            // moveRight suffers from the classic QRect off by one issue
-            popupPosition.moveLeft(bounds.x() + bounds.width() - size.width());
+        if (!inBounds(popupRect, Qt::RightEdge)) {
+            popupRect.moveRight(bounds.right());
         }
     }
     if (constraintAdjustments & PositionerConstraint::ResizeX) {
-        //TODO
-        //but we need to sort out when this is run as resize should only happen before first configure
+        QRect unconstrainedRect = popupRect;
+
+        if (!inBounds(unconstrainedRect, Qt::LeftEdge)) {
+            unconstrainedRect.setLeft(bounds.left());
+        }
+        if (!inBounds(unconstrainedRect, Qt::RightEdge)) {
+            unconstrainedRect.setRight(bounds.right());
+        }
+
+        if (unconstrainedRect.isValid()) {
+            popupRect = unconstrainedRect;
+        }
     }
 
     if (constraintAdjustments & PositionerConstraint::FlipY) {
-        if (!inBounds(popupPosition, Qt::TopEdge | Qt::BottomEdge)) {
+        if (!inBounds(popupRect, Qt::TopEdge | Qt::BottomEdge)) {
             //flip both edges (if either bit is set, XOR both)
             auto flippedAnchorEdge = anchorEdge;
             if (flippedAnchorEdge & (Qt::TopEdge | Qt::BottomEdge)) {
@@ -1729,27 +1692,38 @@ QRect XdgShellClient::transientPlacement(const QRect &bounds) const
             if (flippedGravity & (Qt::TopEdge | Qt::BottomEdge)) {
                 flippedGravity ^= (Qt::TopEdge | Qt::BottomEdge);
             }
-            auto flippedPopupPosition = QRect(popupOffset(anchorRect, flippedAnchorEdge, flippedGravity, size) + offset + parentClientPos, size);
+            auto flippedPopupRect = QRect(popupOffset(anchorRect, flippedAnchorEdge, flippedGravity, size) + offset + parentClientPos, size);
 
             //if it still doesn't fit we should continue with the unflipped version
-            if (inBounds(flippedPopupPosition, Qt::TopEdge | Qt::BottomEdge)) {
-                popupPosition.moveTop(flippedPopupPosition.y());
+            if (inBounds(flippedPopupRect, Qt::TopEdge | Qt::BottomEdge)) {
+                popupRect.moveTop(flippedPopupRect.top());
             }
         }
     }
     if (constraintAdjustments & PositionerConstraint::SlideY) {
-        if (!inBounds(popupPosition, Qt::TopEdge)) {
-            popupPosition.moveTop(bounds.y());
+        if (!inBounds(popupRect, Qt::TopEdge)) {
+            popupRect.moveTop(bounds.top());
         }
-        if (!inBounds(popupPosition, Qt::BottomEdge)) {
-            popupPosition.moveTop(bounds.y() + bounds.height() - size.height());
+        if (!inBounds(popupRect, Qt::BottomEdge)) {
+            popupRect.moveBottom(bounds.bottom());
         }
     }
     if (constraintAdjustments & PositionerConstraint::ResizeY) {
-        //TODO
+        QRect unconstrainedRect = popupRect;
+
+        if (!inBounds(unconstrainedRect, Qt::TopEdge)) {
+            unconstrainedRect.setTop(bounds.top());
+        }
+        if (!inBounds(unconstrainedRect, Qt::BottomEdge)) {
+            unconstrainedRect.setBottom(bounds.bottom());
+        }
+
+        if (unconstrainedRect.isValid()) {
+            popupRect = unconstrainedRect;
+        }
     }
 
-    return popupPosition;
+    return popupRect;
 }
 
 QPoint XdgShellClient::popupOffset(const QRect &anchorRect, const Qt::Edges anchorEdge, const Qt::Edges gravity, const QSize popupSize) const
@@ -1816,7 +1790,7 @@ QMatrix4x4 XdgShellClient::inputTransformation() const
     return matrix;
 }
 
-void XdgShellClient::installServerSideDecoration(KWayland::Server::ServerSideDecorationInterface *deco)
+void XdgShellClient::installServerSideDecoration(KWaylandServer::ServerSideDecorationInterface *deco)
 {
     if (m_serverDecoration == deco) {
         return;
@@ -1881,7 +1855,7 @@ bool XdgShellClient::shouldExposeToWindowManagement()
     return true;
 }
 
-KWayland::Server::XdgShellSurfaceInterface::States XdgShellClient::xdgSurfaceStates() const
+KWaylandServer::XdgShellSurfaceInterface::States XdgShellClient::xdgSurfaceStates() const
 {
     XdgShellSurfaceInterface::States states;
     if (isActive()) {
@@ -1909,12 +1883,6 @@ void XdgShellClient::doMinimize()
     workspace()->updateMinimizedOfTransients(this);
 }
 
-void XdgShellClient::placeIn(const QRect &area)
-{
-    Placement::self()->place(this, area);
-    setGeometryRestore(frameGeometry());
-}
-
 void XdgShellClient::showOnScreenEdge()
 {
     if (!m_plasmaShellSurface || m_unmapped) {
